@@ -1,7 +1,7 @@
 /*
  * abcm2ps: a program to typeset tunes written in abc format using PostScript
  *
- * Copyright (C) 1998-2006 Jean-Franois Moine
+ * Copyright (C) 1998-2007 Jean-Franois Moine
  *
  * Adapted from abc2ps-1.2.5:
  *  Copyright (C) 1996,1997  Michael Methfessel
@@ -48,7 +48,7 @@
 
 /* -- global variables -- */
 
-struct ISTRUCT info, default_info;
+INFO info, default_info;
 unsigned char deco_glob[256], deco_tune[256];
 struct SYMBOL *sym;		/* (points to the symbols of the current voice) */
 
@@ -60,8 +60,9 @@ int in_page;
 				/* switches modified by flags: */
 int pagenumbers;		/* write page numbers ? */
 int epsf;			/* for EPSF postscript output */
+int showerror;		/* show the errors */
 
-char outf[STRL1];		/* output file name */
+char outfn[STRL1];		/* output file name */
 int  file_initialized;		/* for output file */
 FILE *fout;			/* output file */
 char *in_fname;			/* current input file name */
@@ -76,8 +77,6 @@ int nwhistle;
 
 /* -- local variables -- */
 
-static int splittune = -1;	/* tune may be splitted */
-static int deco_old;		/* abc2ps decorations */
 static int def_fmt_done = 0;	/* default format read */
 static struct SYMBOL notitle;
 
@@ -122,18 +121,22 @@ int main(int argc,
 		usage();
 
 	/* initialize */
-	outf[0] = '\0';
-	clrarena(0);			/* global desription */
+	outfn[0] = '\0';
+	clrarena(0);			/* global description */
 	clrarena(1);			/* tune description */
 	clrarena(2);			/* tune generation */
 	clrarena(3);			/* line generation */
 	clear_buffer();
 	abc_init((void *(*)(int size)) getarena, /* alloc */
-		 0,				/* free */
+		0,				/* free */
 		(void (*)(int level)) lvlarena, /* new level */
-		 sizeof(struct SYMBOL) - sizeof(struct abcsym),
-		 0);			/* don't keep comments */
+		sizeof(struct SYMBOL) - sizeof(struct abcsym),
+		0);				/* don't keep comments */
+	memset(&info, 0, sizeof info);
+	info['T' - 'A'] = &notitle;
+	notitle.as.text = "T:";
 	set_format();
+	memcpy(&default_info, &info, sizeof default_info);
 	s_argc = argc;
 	s_argv = argv;
 
@@ -167,6 +170,9 @@ int main(int argc,
 					cfmt.graceslurs = 1;
 					lock_fmt(&cfmt.graceslurs);
 					break;
+				case 'i':
+					showerror = 0;
+					break;
 				case 'j':
 				case 'k':
 					cfmt.measurenb = -1;
@@ -186,7 +192,7 @@ int main(int argc,
 					lock_fmt(&cfmt.writehistory);
 					break;
 				case 'O':
-					outf[0] = '\0';
+					outfn[0] = '\0';
 					break;
 				case 'Q':
 					cfmt.printtempo = 0;
@@ -197,7 +203,10 @@ int main(int argc,
 					lock_fmt(&cfmt.withxrefs);
 					break;
 				case 'W': nwhistle = 0; break;
-				case '0': splittune = 0; break;
+				case '0':
+					cfmt.splittune = 0;
+					lock_fmt(&cfmt.splittune);
+					break;
 				case '1':
 					cfmt.oneperpage = 0;
 					lock_fmt(&cfmt.oneperpage);
@@ -256,6 +265,9 @@ int main(int argc,
 					print_format();
 					return 0;
 				case 'h': usage(); break;
+				case 'i':
+					showerror = 1;
+					break;
 				case 'l':
 					cfmt.landscape = 1;
 					lock_fmt(&cfmt.landscape);
@@ -272,7 +284,10 @@ int main(int argc,
 					cfmt.printtempo = 1;
 					lock_fmt(&cfmt.printtempo);
 					break;
-				case 'u': deco_old = 1; break;
+				case 'u':
+					cfmt.abc2pscompat = 1;
+					lock_fmt(&cfmt.abc2pscompat);
+					break;
 				case 'V':
 					write_version();
 					return 0;
@@ -280,7 +295,10 @@ int main(int argc,
 					cfmt.withxrefs = 1;
 					lock_fmt(&cfmt.withxrefs);
 					break;
-				case '0': splittune = 1; break;
+				case '0':
+					cfmt.splittune = 1;
+					lock_fmt(&cfmt.splittune);
+					break;
 				case '1':
 					cfmt.oneperpage = 1;
 					lock_fmt(&cfmt.oneperpage);
@@ -334,7 +352,8 @@ int main(int argc,
 					aaa = p + 1;
 					if (*aaa == '\0') {
 						aaa = *++argv;
-						if (--argc <= 0 || *aaa == '-') {
+						if (--argc <= 0
+						    || (*aaa == '-' && c != 'O')) {
 							fprintf(stderr,
 								"++++ Missing parameter after flag -%c\n",
 								c);
@@ -437,7 +456,7 @@ int main(int argc,
 						}
 						break;
 					case 'O':
-						strcpy(outf, aaa);
+						strcpy(outfn, aaa);
 						break;
 					case 's':
 						sscanf(aaa, "%f", &cfmt.scale);
@@ -516,11 +535,8 @@ static void output_file(char *sel)
 	/* initialize if not already done */
 	if (fout == 0)
 		set_page_format();
-	memset(&default_info, 0, sizeof default_info);
-	default_info.title = &notitle;
-	notitle.as.text = "T:(notitle)";
 	memcpy(&info, &default_info, sizeof info);
-	reset_deco(deco_old);
+	reset_deco();
 	memcpy(&deco_tune, &deco_glob, sizeof deco_tune);
 	if (!epsf)
 		open_output_file();
@@ -551,8 +567,8 @@ static void do_filter(struct abctune *t, char *sel)
 			sel++;
 			end_sel = (int) ((unsigned) (~0) >> 1);
 			if (sscanf(sel, "%d%n", &end_sel, &n) != 1)
-				break;
-			sel += n;
+				end_sel = 65000;
+			else	sel += n;
 		} else	end_sel = cur_sel;
 		do_select(t, cur_sel, end_sel);
 		if (*sel != ',')
@@ -694,21 +710,22 @@ static void usage(void)
 		"     -E      produce EPSF output, one tune per file\n"
 		"     -O fff  set outfile name to fff\n"
 		"     -O =    make outfile name from infile/title\n"
+		"     -i      indicate where are the errors\n"
 		"  .output formatting:\n"
 		"     -s xx   set scale factor to xx\n"
 		"     -w xx   set staff width (cm/in/pt)\n"
 		"     -m xx   set left margin (cm/in/pt)\n"
 		"     -d xx   set staff separation (cm/in/pt)\n"
 		"     -a xx   set max shrinkage to xx (between 0 and 1)\n"
-		"     -F foo  read format from \"foo.fmt\"\n"
+		"     -F foo  read format file \"foo.fmt\"\n"
 		"     -D bar  look for format files in directory \"bar\"\n"
 		"  .output options:\n"
 		"     -l      landscape mode\n"
 		"     -I xx   indent 1st line (cm/in/pt)\n"
-		"     -x      include xref numbers in output\n"
+		"     -x      add xref numbers in titles\n"
 		"     -M      don't output the lyrics\n"
 		"     -n      include notes and history in output\n"
-		"     -N n    set page number mode to n=\n"
+		"     -N n    set page numbering mode to n=\n"
 		"             0=off 1=left 2=right 3=even left,odd right 4=even right,odd left\n"
 		"     -1      write one tune per page\n"
 		"     -G      no slur in grace notes\n"
@@ -724,7 +741,7 @@ static void usage(void)
 		"  .input file selection/options:\n"
 		"     -e pattern\n"
 		"             xref list of tunes to select\n"
-		"     -u      abc2ps implicit decorations\n"
+		"     -u      abc2ps behaviour\n"
 		"     -L n    set char encoding to Latin number n\n"
 		"  .help/configuration:\n"
 		"     -V      show program version\n"
@@ -814,21 +831,15 @@ static void write_version(void)
 #ifdef A4_FORMAT
 		" A4_FORMAT"
 #endif
-#ifdef CLEF_TRANSPOSE
-		" CLEF_TRANSPOSE"
-#endif
 #ifdef DECO_IS_ROLL
 		" DECO_IS_ROLL"
 #endif
-#if !defined(A4_FORMAT) && !defined(CLEF_TRANSPOSE) \
-     && !defined(DECO_IS_ROLL)
+#if !defined(A4_FORMAT) && !defined(DECO_IS_ROLL)
 		" NONE"
 #endif
 		"\n");
-
 	if (strlen(styd) > 0)
-		fprintf(stderr,
-			"Default format directory: %s\n", styd);
+		fprintf(stderr, "Default format directory: %s\n", styd);
 }
 
 /* -- arena routines -- */
