@@ -1,7 +1,7 @@
 /*++
  * Generic ABC parser.
  *
- * Copyright (C) 1998-2006 Jean-Franois Moine
+ * Copyright (C) 1998-2007 Jean-Franois Moine
  * Adapted from abc2ps, Copyright (C) 1996, 1997  Michael Methfessel
  *
  * Original site: http://moinejf.free.fr/
@@ -309,6 +309,7 @@ struct abcsym *abc_new(struct abctune *t,
 		s->prev = t->last_sym;
 		t->last_sym = s;
 	}
+	s->state = abc_state;
 	s->linenum = linenum;
 	s->colnum = colnum;
 	return s;
@@ -598,7 +599,7 @@ static char *parse_acc(char *p,
 }
 
 /* -- parse a clef (K: or V:) -- */
-static void parse_clef(struct clef_s *p_clef,
+static void parse_clef(struct abcsym *s,
 		       char *name,
 		       char *middle,
 		       char *lines,
@@ -652,11 +653,7 @@ static void parse_clef(struct clef_s *p_clef,
 		if (!strncmp(name, "bass", 4)) {
 			clef = BASS;
 			clef_line = 4;
-#ifdef CLEF_TRANSPOSE
-			transpose = -14;
-#else
-			p_clef->check_pitch = 1;
-#endif
+			s->u.clef.check_pitch = 1;
 			name += 4;
 		} else if (!strncmp(name, "treble", 6)) {
 			clef = TREBLE;
@@ -665,11 +662,7 @@ static void parse_clef(struct clef_s *p_clef,
 			   || !strncmp(name, "tenor", 5)) {
 			clef = ALTO;
 			clef_line = *name == 'a' ? 3 : 4;
-#ifdef CLEF_TRANSPOSE
-			transpose = -7;
-#else
-			p_clef->check_pitch = 1;
-#endif
+			s->u.clef.check_pitch = 1;
 			if (*name == 'a')
 				name += 4;
 			else	name += 5;
@@ -678,7 +671,8 @@ static void parse_clef(struct clef_s *p_clef,
 			name += 4;
 		} else if (strncmp(name, "none", 4) == 0) {
 			clef = TREBLE;
-			p_clef->invis = 1;
+			s->u.clef.invis = 1;
+			s->flags |= ABC_F_INVIS;
 			name += 4;
 		} else	syntax("Unknown clef", name);
 	}
@@ -695,9 +689,9 @@ static void parse_clef(struct clef_s *p_clef,
 		}
 		if (name[1] == '8') {
 			if (*name == '-')
-				p_clef->octave = -1;
+				s->u.clef.octave = -1;
 			else if (*name == '+')
-				p_clef->octave = 1;
+				s->u.clef.octave = 1;
 		}
 	}
 
@@ -721,30 +715,28 @@ static void parse_clef(struct clef_s *p_clef,
 		l = l - pit + 4 + 14;
 		clef_line = (l % 7) / 2 + 1;
 		transpose = l / 7 * 7 - 14;
-#ifndef CLEF_TRANSPOSE
-		p_clef->check_pitch = 0;
-#endif
+		s->u.clef.check_pitch = 0;
 	}
 
-	p_clef->type = clef;
-	p_clef->line = clef_line;
-	p_clef->transpose = curvoice->add_pitch = transpose;
-	p_clef->stafflines = -1;
-	p_clef->staffscale = 0;
+	s->u.clef.type = clef;
+	s->u.clef.line = clef_line;
+	s->u.clef.transpose = curvoice->add_pitch = transpose;
+	s->u.clef.stafflines = -1;
+	s->u.clef.staffscale = 0;
 	if (lines != 0) {
 		int l;
 
 		l = atoi(lines);
 		if ((unsigned) l < 10)
-			p_clef->stafflines = l;
+			s->u.clef.stafflines = l;
 		else	syntax("Bad value of stafflines", lines);
 	}
 	if (scale != 0) {
 		float sc;
 
 		sc = atof(scale);
-		if (sc >= 0.5 && sc <= 2)
-			p_clef->staffscale = sc;
+		if (sc >= 0.5 && sc <= 3)
+			s->u.clef.staffscale = sc;
 		else	syntax("Bad value of staffscale", scale);
 	}
 }
@@ -911,7 +903,7 @@ static void parse_key(char *p,
 	if (clef_name != 0 || clef_middle != 0 || clef_lines != 0 || clef_scale != 0) {
 		s = abc_new(s->tune, 0, 0);
 		s->type = ABC_T_CLEF;
-		parse_clef(&s->u.clef, clef_name, clef_middle, clef_lines, clef_scale);
+		parse_clef(s, clef_name, clef_middle, clef_lines, clef_scale);
 		if (abc_state == ABC_S_HEAD) {	/* if first K: */
 			int i;
 
@@ -1061,75 +1053,86 @@ static char *top_err = "Cannot identify meter top";
 	wmeasure = 0;
 	if (*p == 'N' || *p == 'n')
 		p++;				/* no meter */
-	else if (*p == 'C') {
-		s->u.meter.meter[0].top[0] = *p++;
-		if (*p == '|')
-			s->u.meter.meter[0].top[1] = *p++;
-		wmeasure = 4 * BASE_LEN / 4;
-		nm = 1;
-	} else while (*p != '\0') {
+	else while (*p != '\0') {
 		if (*p == '=')
 			break;
 		if (nm >= MAX_MEASURE)
 			return "Too many values in M:";
-		if (*p == '(') {
+		switch (*p) {
+		case 'C':
+			s->u.meter.meter[nm].top[0] = *p++;
+			if (*p == '|')
+				s->u.meter.meter[nm].top[1] = *p++;
+			m1 = 4;
+			m2 = 4;
+			break;
+		case 'c':
+		case 'o':
+			if (*p == 'c')
+				m1 = 4;
+			else	m1 = 3;
+			m2 = 4;
+			s->u.meter.meter[nm].top[0] = *p++;
+			if (*p == '.')
+				s->u.meter.meter[nm].top[1] = *p++;
+			break;
+		case '(':
 			q = p + 1;
 			while (*q != '\0') {
 				if (*q == ')' || *q == '/')
 					break;
 				q++;
 			}
-			if (*q == ')')
-				p++;	/* stay on top */
-		}
-		if (*p == '(' || *p == ')') {
-			in_parenth = *p == '(';
-			s->u.meter.meter[nm].top[0] = *p++;
-			nm++;
-			continue;
-		}
-		if (sscanf(p, "%d", &m1) != 1
-		    || m1 <= 0)
-			return top_err;
-		i = 0;
-		m2 = 2;			/* default when no bottom value */
-		for (;;) {
-			while (isdigit((unsigned) *p)
-			       && i < sizeof s->u.meter.meter[0].top)
-				s->u.meter.meter[nm].top[i++] = *p++;
-			if (*p == ')')
-				p++;
-			if (*p == '/') {
-				p++;
-				if (sscanf(p, "%d", &m2) != 1
-				    || m2 <= 0)
-					return "Cannot identify meter bottom";
-				i = 0;
-				while (isdigit((unsigned) *p)
-				       && i < sizeof s->u.meter.meter[0].bot)
-					s->u.meter.meter[nm].bot[i++] = *p++;
-				break;
+			if (*q == ')') {
+				p++;		/* stay on top */
+				continue;
 			}
-			if (*p != ' ' && *p != '+')
-				break;
-			s->u.meter.meter[nm].top[i++] = *p++;
-			if (sscanf(p, "%d", &d) != 1
-			    || d <= 0)
+		case ')':
+			in_parenth = *p == '(';
+			s->u.meter.meter[nm++].top[0] = *p++;
+			continue;
+		default:
+			if (sscanf(p, "%d", &m1) != 1
+			    || m1 <= 0)
 				return top_err;
-			if (p[-1] == ' ') {
-				if (d > m1)
-					m1 = d;
-			} else	m1 += d;
+			i = 0;
+			m2 = 2;			/* default when no bottom value */
+			for (;;) {
+				while (isdigit((unsigned) *p)
+				       && i < sizeof s->u.meter.meter[0].top)
+					s->u.meter.meter[nm].top[i++] = *p++;
+				if (*p == ')')
+					p++;
+				if (*p == '/') {
+					p++;
+					if (sscanf(p, "%d", &m2) != 1
+					    || m2 <= 0)
+						return "Cannot identify meter bottom";
+					i = 0;
+					while (isdigit((unsigned) *p)
+					       && i < sizeof s->u.meter.meter[0].bot)
+						s->u.meter.meter[nm].bot[i++] = *p++;
+					break;
+				}
+				if (*p != ' ' && *p != '+')
+					break;
+				s->u.meter.meter[nm].top[i++] = *p++;
+				if (sscanf(p, "%d", &d) != 1
+				    || d <= 0)
+					return top_err;
+				if (p[-1] == ' ') {
+					if (d > m1)
+						m1 = d;
+				} else	m1 += d;
+			}
 		}
 		if (!in_parenth)
 			wmeasure += m1 * BASE_LEN / m2;
 		nm++;
 		if (*p == ' ')
 			p++;
-		else if (*p == '+') {
-			s->u.meter.meter[nm].top[0] = *p++;
-			nm++;
-		}
+		else if (*p == '+')
+			s->u.meter.meter[nm++].top[0] = *p++;
 	}
 	if (*p == '=') {
 		if (sscanf(++p, "%d/%d", &m1, &m2) != 2
@@ -1139,7 +1142,6 @@ static char *top_err = "Cannot identify meter top";
 		wmeasure = m1 * BASE_LEN / m2;
 		s->u.meter.expdur = 1;
 	}
-
 	s->u.meter.wmeasure = wmeasure;
 	s->u.meter.nmeter = nm;
 
@@ -1150,57 +1152,57 @@ static char *top_err = "Cannot identify meter top";
 			ulen = BASE_LEN / 8;
 		else	ulen = BASE_LEN / 16;
 	}
-
 	return 0;
 }
 
 /* -- treat %%staves -- */
 static void parse_staves(char *p,
-			 struct abcsym *s)
+			 struct abcsym *s,
+			 int score)
 {
-	int voice, flags_st;
-	char flags, flags2;
+	int voice, flags_st, brace, bracket, parenth;
+	char flags[2];
 	struct staff_s *p_staff;
 
 	/* define the voices */
-	flags = flags2 = 0;
+	flags[0] = score ? STOP_BAR : 0;
+	flags[1] = 0;
+	brace = bracket = parenth = 0;
 	flags_st = 0;
 	voice = 0;
-
 	while (*p != '\0') {
 		switch (*p) {
 		case ' ':
 		case '\t':
 			break;
 		case '[':
-#if 1
-/*fixme: should limit at 2 open bracket*/
-			if (flags2 & OPEN_PARENTH)
-#else
-			if (flags2 & (OPEN_BRACKET | OPEN_PARENTH))
-#endif
+			if (parenth || brace + bracket >= 2)
 				goto err;
-			flags2 |= OPEN_BRACKET;
-			flags |= OPEN_BRACKET;
+			flags[brace + bracket] |= OPEN_BRACKET;
+			bracket++;
 			flags_st <<= 8;
 			flags_st |= OPEN_BRACKET;
 			break;
 		case '{':
-			if (flags2 & (OPEN_BRACE | OPEN_PARENTH))
+			if (parenth || brace || bracket >= 2)
 				goto err;
-			flags2 |= OPEN_BRACE;
-			flags |= OPEN_BRACE;
+			flags[bracket] |= OPEN_BRACE;
+			brace++;
 			flags_st <<= 8;
 			flags_st |= OPEN_BRACE;
 			break;
 		case '(':
-			if (flags2 & OPEN_PARENTH)
+			if (parenth)
 				goto err;
-			flags2 |= OPEN_PARENTH;
-			flags |= OPEN_PARENTH;
+			flags[0] |= OPEN_PARENTH;
+			parenth++;
 			flags_st <<= 8;
 			flags_st |= OPEN_PARENTH;
 			break;
+		case '*':
+			flags[0] |= FL_VOICE;
+			p++;
+			break;
 		default:
 			if (!isalnum((unsigned) *p) && *p != '_')
 				goto err;
@@ -1214,8 +1216,6 @@ static void parse_staves(char *p,
 				p = def_voice(p, &v);
 				p_staff = &s->u.staves[voice++];
 				p_staff->voice = v;
-				p_staff->name = alloc_f(strlen(voice_tb[v].name) + 1);
-				strcpy(p_staff->name, voice_tb[v].name);
 			}
 			for (;;) {
 				switch (*p) {
@@ -1226,36 +1226,40 @@ static void parse_staves(char *p,
 				case ']':
 					if (!(flags_st & OPEN_BRACKET))
 						goto err;
+					bracket--;
+					flags[brace + bracket] |= CLOSE_BRACKET;
 					flags_st >>= 8;
-					flags2 &= ~OPEN_BRACKET;
-					flags |= CLOSE_BRACKET;
 					p++;
 					continue;
 				case '}':
 					if (!(flags_st & OPEN_BRACE))
 						goto err;
+					brace--;
+					flags[bracket] |= CLOSE_BRACE;
 					flags_st >>= 8;
-					flags2 &= ~OPEN_BRACE;
-					flags |= CLOSE_BRACE;
 					p++;
 					continue;
 				case ')':
 					if (!(flags_st & OPEN_PARENTH))
 						goto err;
+					parenth--;
+					flags[0] |= CLOSE_PARENTH;
 					flags_st >>= 8;
-					flags2 &= ~OPEN_PARENTH;
-					flags |= CLOSE_PARENTH;
 					p++;
 					continue;
 				case '|':
-					flags |= STOP_BAR;
+					if (!score)
+						flags[0] |= STOP_BAR;
+					else	flags[0] &= ~STOP_BAR;
 					p++;
 					continue;
 				}
 				break;
 			}
-			p_staff->flags = flags;
-			flags = 0;
+			p_staff->flags[0] = flags[0];
+			p_staff->flags[1] = flags[1];
+			flags[0] = score ? STOP_BAR : 0;
+			flags[1] = 0;
 			if (*p == '\0')
 				break;
 			continue;
@@ -1264,11 +1268,25 @@ static void parse_staves(char *p,
 			break;
 		p++;
 	}
-	if (flags2 == 0)
-		return;
-
+	if (flags_st == 0)
+		goto ok;
 err:
 	syntax("%%staves error", p);
+	{
+		int i;
+
+		for (i = 0; i < voice; i++) {
+			s->u.staves[i].flags[0] &= ~(OPEN_PARENTH | CLOSE_PARENTH
+						| OPEN_BRACKET | CLOSE_BRACKET
+						| OPEN_BRACE | OPEN_BRACE);
+			s->u.staves[i].flags[1] &= ~(OPEN_BRACKET | CLOSE_BRACKET
+						| OPEN_BRACE | OPEN_BRACE);
+		}
+	}
+	s->flags |= ABC_F_ERROR;
+ok:
+	if (voice < MAXVOICE)
+		s->u.staves[voice].voice = -1;
 }
 
 /* -- get a possibly quoted string -- */
@@ -1276,29 +1294,35 @@ char *get_str(char *d,		/* destination */
 	      char *s,		/* source */
 	      int maxlen)	/* max length */
 {
-	char sep, c;
+	char c;
 
 	maxlen--;		/* have place for the EOS */
 	while (isspace((unsigned) *s))
 		s++;
 	if (*s == '"') {
-		sep = '"';
 		s++;
-	} else	sep = ' ';
-	while ((c = *s) != '\0') {
-		if (c == sep
-		    || (c == '\t' && sep == ' ')) {
-			if (sep != ' ')
+		while ((c = *s) != '\0') {
+			if (c == '"') {
 				s++;
-			break;
+				break;
+			}
+			if (c == '\\') {
+				if (--maxlen > 0)
+					*d++ = c;
+				c = *++s;
+			}
+			if (--maxlen > 0)
+				*d++ = c;
+			s++;
+		}
+	} else {
+		while ((c = *s) != '\0') {
+			if (isspace(c))
+				break;
+			if (--maxlen > 0)
+				*d++ = c;
+			s++;
 		}
-		if (c == '\\'
-		    && (s[1] == sep
-		       || (s[1] == '\t' && sep == ' ')))
-			c = *++s;
-		if (--maxlen > 0)
-			*d++ = c;
-		s++;
 	}
 	*d = '\0';
 	while (isspace((unsigned) *s))
@@ -1311,21 +1335,13 @@ static char *parse_tempo(char *p,
 			 struct abcsym *s)
 {
 	int l, have_error = 0;
-	char *q;
+	char *q, str[80];
 
 	/* string before */
 	if (*p == '"') {
-		q = ++p;
-		while (*p != '"' && *p != '\0')
-			p++;
-		l = p - q;
-		s->u.tempo.str1 = alloc_f(l + 1);
-		strncpy(s->u.tempo.str1, q, l);
-		s->u.tempo.str1[l] = '\0';
-		if (*p == '"')
-			p++;
-		while (isspace((unsigned) *p))
-			p++;
+		p = get_str(str, p, sizeof str);
+		s->u.tempo.str1 = alloc_f(strlen(str) + 1);
+		strcpy(s->u.tempo.str1, str);
 	}
 
 	/* beat */
@@ -1385,15 +1401,10 @@ static char *parse_tempo(char *p,
 
 	/* string after */
 	if (*p == '"') {
-		q = ++p;
-		while (*p != '"' && *p != '\0')
-			p++;
-		l = p - q;
-		s->u.tempo.str2 = alloc_f(l + 1);
-		strncpy(s->u.tempo.str2, q, l);
-		s->u.tempo.str2[l] = '\0';
+		p = get_str(str, p, sizeof str);
+		s->u.tempo.str2 = alloc_f(strlen(str) + 1);
+		strcpy(s->u.tempo.str2, str);
 	}
-
 	return have_error ? "Invalid tempo" : 0;
 }
 
@@ -1549,7 +1560,7 @@ static struct kw_s {
 	if (clef_name != 0 || clef_middle != 0 || clef_lines != 0 || clef_scale != 0) {
 		s = abc_new(s->tune, 0, 0);
 		s->type = ABC_T_CLEF;
-		parse_clef(&s->u.clef, clef_name, clef_middle, clef_lines, clef_scale);
+		parse_clef(s, clef_name, clef_middle, clef_lines, clef_scale);
 	}
 	return error_txt;
 }
@@ -1607,7 +1618,6 @@ static char *parse_bar(struct abctune *t,
 		gchord = 0;
 	}
 	s->type = ABC_T_BAR;
-	s->state = ABC_S_TUNE;
 	s->u.bar.type = bar_type;
 
 	if (dc.n > 0) {
@@ -1638,7 +1648,6 @@ static char *parse_bar(struct abctune *t,
 	    || s->text != 0) {
 		s = abc_new(t, repeat_value, 0);
 		s->type = ABC_T_BAR;
-		s->state = ABC_S_TUNE;
 		s->u.bar.type = B_OBRA;
 	} else {
 		s->text = alloc_f(strlen(repeat_value) + 1);
@@ -1853,7 +1862,7 @@ static char *parse_decoline(char *p)
 
 		/* store the decoration / gchord/annotation in the next note */
 		while (is != 0 && (is->type != ABC_T_NOTE
-				   || is->u.note.grace))
+				   || (is->flags & ABC_F_GRACE)))
 			is = is->next;
 		if (is == 0)
 			return "Not enough notes for deco line";
@@ -1899,6 +1908,8 @@ static char *parse_gchord(char *p,
 	more_gch = 0;
 	q = p;
 	while (*p != '"') {
+		if (*p == '\\')
+			p++;
 		if (*p == '\0') {
 			more_gch = 1;
 			break;
@@ -1971,6 +1982,7 @@ static int parse_line(struct abctune *t,
 	struct abcsym *s;
 	char *comment, *q, c;
 	struct abcsym *last_note_sav = 0;
+	struct deco dc_sav;
 	int i;
 	char sappo = 0;
 	static char qtb[10] = {0, 1, 3, 2, 3, 0, 2, 0, 3, 0};
@@ -1985,7 +1997,6 @@ again:					/* for history */
 			if (keep_comment) {
 				s = abc_new(t, 0, 0);
 				s->type = ABC_T_NULL;
-				s->state = abc_state;
 			}
 			return 0;
 		}
@@ -2002,7 +2013,6 @@ again:					/* for history */
 			comment = decomment_line(p + 2);
 			s = abc_new(t, p, comment);
 			s->type = ABC_T_PSCOM;
-			s->state = abc_state;
 			p += 2;				/* skip '%%' */
 			while (isspace((unsigned) *p))
 				p++;
@@ -2029,7 +2039,6 @@ again:					/* for history */
 					}
 					s = abc_new(t, p, 0);
 					s->type = ABC_T_PSCOM;
-					s->state = abc_state;
 					if (*p != '%' || p[1] != '%')
 						continue;
 					p += 2;
@@ -2051,7 +2060,9 @@ again:					/* for history */
 				/* not reached */
 			}
 			if (strncmp(p, "staves ", 7) == 0)
-				parse_staves(p + 7, s);
+				parse_staves(p + 7, s, 0);
+			else if (strncmp(p, "score ", 6) == 0)
+				parse_staves(p + 6, s, 1);
 			return 0;
 		}
 		/* fall thru */
@@ -2059,7 +2070,6 @@ again:					/* for history */
 		if (keep_comment) {
 			s = abc_new(t, p, 0);
 			s->type = ABC_T_NULL;
-			s->state = abc_state;
 		}
 		return 0;		/* skip */
 	}
@@ -2073,15 +2083,17 @@ again:					/* for history */
 
 			/* loop till any 'x:' or '%%' */
 			for (;;) {
-				if ((p = get_line()) == 0)
-					break;
+				if ((p = get_line()) == 0) {
+					syntax("EOF while parsing H:",
+					       scratch_line);
+					return 1;
+				}
 				if (p[1] == ':'
 				    || (p[1] == '%' && *p == '%'))
 					goto again;
 				if (abc_state == ABC_S_HEAD) {
 					s = abc_new(t, p, 0);
 					s->type = ABC_T_INFO2;
-					s->state = abc_state;
 				}
 			}
 		}
@@ -2103,13 +2115,12 @@ again:					/* for history */
 		if (keep_comment) {
 			s = abc_new(t, p, comment);
 			s->type = ABC_T_NULL;
-			s->state = abc_state;
 		}
 		return 0;
 	}
 
-	if (isspace((unsigned) scratch_line[0]) && curvoice->last_note != 0)
-		curvoice->last_note->u.note.word_end = 1;
+	if (isspace((unsigned) scratch_line[0]))
+		t->last_sym->flags |= ABC_F_WORD_END;
 
 	lyric_started = 0;
 	deco_start = deco_cont = 0;
@@ -2126,7 +2137,7 @@ again:					/* for history */
 				if ((p = get_line()) == 0) {
 					syntax("EOF reached while parsing guitar chord",
 					       p);
-					break;
+					return 1;
 				}
 			}
 			break;
@@ -2141,12 +2152,15 @@ again:					/* for history */
 				char_tb['}'] = CHAR_GRACE;
 				last_note_sav = curvoice->last_note;
 				curvoice->last_note = 0;
+				memcpy(&dc_sav, &dc, sizeof dc);
+				dc.n = dc.h = dc.s = 0;
 			} else {
 				char_tb['{'] = CHAR_GRACE;
 				char_tb['}'] = CHAR_BAD;
 /*fixme:bad*/
-				t->last_sym->u.note.word_end = 1;
+				t->last_sym->flags |= (ABC_F_WORD_END | ABC_F_GR_END);
 				curvoice->last_note = last_note_sav;
+				memcpy(&dc, &dc_sav, sizeof dc);
 			}
 			break;
 		case CHAR_DECO:
@@ -2154,7 +2168,7 @@ again:					/* for history */
 			if (p[-1] == '!' && check_nl(p)) {
 				s = abc_new(t, 0, 0);	/* abc2win EOL */
 				s->type = ABC_T_EOLN;
-				s->state = abc_state;
+				s->u.eoln.type = 2;
 				break;
 			}
 			if (p[-1] == '.') {
@@ -2182,10 +2196,11 @@ again:					/* for history */
 		case CHAR_REST:
 			p = parse_note(t, p - 1);
 			if (sappo) {
-				t->last_sym->u.note.sappo = 1;
+				t->last_sym->flags |= ABC_F_SAPPO;
 				sappo = 0;
 			}
-			curvoice->last_note = t->last_sym;
+			if (t->last_sym->u.note.lens[0] > 0)	/* if not space */
+				curvoice->last_note = t->last_sym;
 			break;
 		case CHAR_SLASH:		/* '/' */
 			q = p;
@@ -2193,18 +2208,22 @@ again:					/* for history */
 				q++;
 			if (char_tb[(unsigned) *q] != CHAR_BAR)
 				goto bad_char;
-			syntax("Obsolete measure repeat syntax", p - 1);
+			syntax("Non standard measure repeat syntax", p - 1);
 			s = abc_new(t, 0, 0);
 			s->type = ABC_T_MREP;
-			s->state = ABC_S_TUNE;
 			s->u.bar.type = 0;
 			s->u.bar.len = q - p + 1;
+			s->flags |= ABC_F_ERROR;
 			p = q;
 			break;
 		case CHAR_BSLASH:		/* '\\' */
 /*fixme: KO if in grace note sequence*/
-			if (*p == '\0')
+			if (*p == '\0') {
+				s = abc_new(t, 0, 0);	/* abc2win EOL */
+				s->type = ABC_T_EOLN;
+				s->u.eoln.type = 1;
 				return 0;
+			}
 			syntax("'\\' ignored", p - 1);
 			break;
 		case CHAR_OBRA:			/* '[' */
@@ -2216,7 +2235,7 @@ again:					/* for history */
 			if (p[1] != ':') {
 				p = parse_note(t, p - 1);	/* chord */
 				if (sappo) {
-					t->last_sym->u.note.sappo = 1;
+					t->last_sym->flags |= ABC_F_SAPPO;
 					sappo = 0;
 				}
 				curvoice->last_note = t->last_sym;
@@ -2280,12 +2299,11 @@ again:					/* for history */
 					qplet = meter % 3 == 0 ? 3 : 2;
 				s = abc_new(t, 0, 0);
 				s->type = ABC_T_TUPLET;
-				s->state = ABC_S_TUNE;
 				s->u.tuplet.p_plet = pplet;
 				s->u.tuplet.q_plet = qplet;
 				s->u.tuplet.r_plet = rplet;
 				if (char_tb['{'] == CHAR_BAD)
-					s->u.tuplet.grace = 1;
+					s->flags |= ABC_F_GRACE;
 				break;
 			}
 			if (*p == '&') {
@@ -2329,6 +2347,10 @@ again:					/* for history */
 		case CHAR_VOV:			/* '&' */
 			if (*p != ')'
 			    || vover == 0) {		/*??*/
+				if (curvoice->last_note == 0) {
+					syntax("Bad start of voice overlay", p);
+					break;
+				}
 				s = abc_new(t, 0, 0);
 				s->type = ABC_T_V_OVER;
 				/*s->u.v_over.type = V_OVER_V; */
@@ -2348,8 +2370,7 @@ again:					/* for history */
 			curvoice = &voice_tb[curvoice->mvoice];
 			break;
 		case CHAR_SPAC:			/* ' ' and '\t' */
-			if (curvoice->last_note != 0)
-				curvoice->last_note->u.note.word_end = 1;
+			t->last_sym->flags |= ABC_F_WORD_END;
 			break;
 		case CHAR_MINUS: {		/* '-' */
 			int tie_pos;
@@ -2375,7 +2396,7 @@ again:					/* for history */
 					curvoice->last_note->u.note.ti1[i] = tie_pos;
 			}
 			if (curvoice->dotted_tie) {
-				curvoice->last_note->u.note.dotted_tie = 1;
+				curvoice->last_note->flags |= ABC_F_DOTTED_TIE;
 				curvoice->dotted_tie = 0;
 			}
 			break;
@@ -2412,14 +2433,16 @@ again:					/* for history */
 		char_tb['{'] = CHAR_GRACE;
 		char_tb['}'] = CHAR_BAD;
 		if (curvoice->last_note != 0)
-			curvoice->last_note->u.note.word_end = 1;
+			curvoice->last_note->flags |=
+				(ABC_F_WORD_END | ABC_F_GR_END | ABC_F_ERROR);
 		curvoice->last_note = last_note_sav;
+		memcpy(&dc, &dc_sav, sizeof dc);
 	}
 
 	/* add eoln */
 	s = abc_new(t, 0, 0);
 	s->type = ABC_T_EOLN;
-	s->state = abc_state;
+/*	s->as.u.eoln = 0; */
 	return 0;
 }
 
@@ -2433,7 +2456,7 @@ static char *parse_note(struct abctune *t,
 
 	if (char_tb['{'] == CHAR_BAD) {	/* in a grace note sequence */
 		s = abc_new(t, 0, 0);
-		s->u.note.grace = 1;
+		s->flags |= ABC_F_GRACE;
 	} else {
 		s = abc_new(t, gchord, 0);
 		if (gchord) {
@@ -2443,11 +2466,10 @@ static char *parse_note(struct abctune *t,
 		}
 	}
 	s->type = ABC_T_NOTE;
-	s->state = ABC_S_TUNE;
 
-	if (*p != 'Z' && !lyric_started && !s->u.note.grace) {
+	if (*p != 'Z' && !lyric_started && !(s->flags & ABC_F_GRACE)) {
 		lyric_started = 1;
-		s->u.note.lyric_start = 1;
+		s->flags |= ABC_F_LYRIC_START;
 		deco_start = s;
 	}
 
@@ -2466,7 +2488,7 @@ static char *parse_note(struct abctune *t,
 		goto add_deco;
 	case 'y':			/* space (BarFly) */
 		s->type = ABC_T_REST;
-		s->u.note.invis = 1;
+		s->flags |= ABC_F_INVIS;
 		p++;
 		if (isdigit((unsigned) *p)) {	/* number of points */
 			s->u.note.lens[1] = strtol(p, &q, 10);
@@ -2474,7 +2496,7 @@ static char *parse_note(struct abctune *t,
 		} else	s->u.note.lens[1] = -1;
 		goto add_slurs;
 	case 'x':			/* invisible rest */
-		s->u.note.invis = 1;
+		s->flags |= ABC_F_INVIS;
 		/* fall thru */
 	case 'z':
 		s->type = ABC_T_REST;
@@ -2498,7 +2520,7 @@ static char *parse_note(struct abctune *t,
 
 		if (chord) {
 			if (*p == '.' && p[1] == '(') {
-				s->u.note.dotted_slur = 1;
+				s->flags |= ABC_F_DOTTED_SLUR;
 				p++;
 			}
 			if (*p == '(') {
@@ -2524,6 +2546,7 @@ static char *parse_note(struct abctune *t,
 		if (dc.n != tmp) {
 			if (dc.n - tmp >= 8) {
 				syntax("Too many decorations on this head", p);
+				s->flags |= ABC_F_ERROR;
 				tmp = dc.n - 7;
 			}
 			s->u.note.decs[m] = (tmp << 3) + dc.n - tmp;
@@ -2531,11 +2554,12 @@ static char *parse_note(struct abctune *t,
 		}
 		if (*p == '\0' || strchr(all_notes, *p) == 0) {
 			syntax("Not a note", p);
+			s->flags |= ABC_F_ERROR;
 			if (*p != '\0')
 				p++;
 		} else {
 			p = parse_basic_note(p, &pit, &len, &acc, &tmp);
-			if (s->u.note.grace) {
+			if (s->flags & ABC_F_GRACE) {
 				len = len * BASE_LEN / 4 / ulen;
 				tmp = 0;
 			}
@@ -2548,7 +2572,7 @@ static char *parse_note(struct abctune *t,
 				for (;;) {
 					if (*p == '.') {
 						if (p[1] == '-')
-							s->u.note.dotted_tie = 1;
+							s->flags |= ABC_F_DOTTED_TIE;
 						else if (p[1] == '-')
 							;
 						else	break;
@@ -2597,10 +2621,12 @@ static char *parse_note(struct abctune *t,
 		}
 		if (*p == '\0') {
 			syntax("Chord not closed", q);
+			s->flags |= ABC_F_ERROR;
 			break;
 		}
 	}
-	s->u.note.stemless = nostem;
+	if (nostem)
+		s->flags |= ABC_F_STEMLESS;
 
 	if (m == 0) {			/* if no note */
 		if ((t->last_sym = s->prev) == 0)
@@ -2618,12 +2644,14 @@ add_slurs:
 	s->u.note.slur_st = curvoice->slur;
 	curvoice->slur = 0;
 	if (curvoice->dotted_slur) {
-		s->u.note.dotted_slur = 1;
+		s->flags |= ABC_F_DOTTED_SLUR;
 		curvoice->dotted_slur = 0;
 	}
 add_deco:
 	if (dc.n > 0) {
-		memcpy(&s->u.note.dc, &dc, sizeof s->u.note.dc);
+		memcpy(s->type != ABC_T_MREST ? &s->u.note.dc
+				: &s->u.bar.dc,
+			&dc, sizeof dc);
 		dc.n = dc.h = dc.s = 0;
 	}
 	return p;
@@ -2640,7 +2668,6 @@ static void parse_info(struct abctune *t,
 
 	s = abc_new(t, p, comment);
 	s->type = ABC_T_INFO;
-	s->state = abc_state;
 
 	p += 2;
 	while (isspace((unsigned) *p))
@@ -2710,8 +2737,10 @@ static void parse_info(struct abctune *t,
 			level_f(1);
 		break;
 	}
-	if (error_txt != 0)
+	if (error_txt != 0) {
 		syntax(error_txt, p);
+		s->flags |= ABC_F_ERROR;
+	}
 }
 
 /* -- print a syntax error message -- */
