@@ -3,7 +3,7 @@
  *
  * This file is part of abcm2ps.
  *
- * Copyright (C) 2000-2006, Jean-Franois Moine.
+ * Copyright (C) 2000-2007, Jean-Franois Moine.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -68,27 +68,27 @@ static struct deco_def_s {
 	unsigned char str;	/* string index - 255=deco name */
 } deco_def_tb[128] = {
 	{0,		0, 0, 0},	/* 0: unknown */
-	{"dot",		0, 0, 2},	/* 1 */
-	{"roll",	3, 1, 10},	/* 2 */
-	{"fermata",	3, 2, 12},	/* 3 */
-	{"emphasis",	3, 3, 8},	/* 4 */
-	{"lowermordent", 3, 4, 10},	/* 5 */
-	{"coda",	3, 5, 24},	/* 6 */
-	{"uppermordent", 3, 6, 10},	/* 7 */
-	{"segno",	3, 7, 20},	/* 8 */
-	{"trill",	3, 8, 11},	/* 9 */
-	{"upbow",	3, 9, 10},	/* 10 */
-	{"downbow",	3, 10, 9},	/* 11 */
-	{"gmark",	3, 11, 6},	/* 12 */
-	{"slide",	1, 12, 3, 7},	/* 13 */
-	{"tenuto",	0, 13, 2},	/* 14 */
-	{"breath",	3, 14, 0},	/* 15 */
-	{"longphrase",	3, 15, 0},	/* 16 */
-	{"mediumphrase", 3, 16, 0},	/* 17 */
-	{"shortphrase", 3, 17, 0},	/* 18 */
-	{"invertedfermata", 3, 2, 12},	/* 19 */
-	{"invertedturn", 3, 18, 10},	/* 20 */
-	{"invertedturnx", 3, 19, 10},	/* 21 */
+	{"dot",		0, 0, 5, 1, 1},	/* 1 */
+	{"roll",	3, 1, 10, 6, 6}, /* 2 */
+	{"fermata",	3, 2, 12, 7, 7}, /* 3 */
+	{"emphasis",	3, 3, 8, 4, 4},	/* 4 */
+	{"lowermordent", 3, 4, 10, 2, 2}, /* 5 */
+	{"coda",	3, 5, 24, 10, 10}, /* 6 */
+	{"uppermordent", 3, 6, 10, 2, 2}, /* 7 */
+	{"segno",	3, 7, 20, 4, 4}, /* 8 */
+	{"trill",	3, 8, 11, 4, 4}, /* 9 */
+	{"upbow",	3, 9, 10, 5, 5}, /* 10 */
+	{"downbow",	3, 10, 9, 5, 5}, /* 11 */
+	{"gmark",	3, 11, 6, 5, 5}, /* 12 */
+	{"slide",	1, 12, 3, 7, 0}, /* 13 */
+	{"tenuto",	0, 13, 5, 2, 2}, /* 14 */
+	{"breath",	3, 14, 0, 1, 20}, /* 15 */
+	{"longphrase",	3, 15, 0, 1, 1}, /* 16 */
+	{"mediumphrase", 3, 16, 0, 1, 1}, /* 17 */
+	{"shortphrase", 3, 17, 0, 1, 1}, /* 18 */
+	{"invertedfermata", 3, 2, 12, 7, 7}, /* 19 */
+	{"invertedturn", 3, 18, 10, 0, 5}, /* 20 */
+	{"invertedturnx", 3, 19, 10, 0, 5}, /* 21 */
 };
 
 /* c function table */
@@ -130,40 +130,40 @@ static char *ps_func_tb[128] = {
 static char *str_tb[32];
 
 static char *std_deco_tb[] = {
-	"0 3 fng 8 0 0 0",
-	"1 3 fng 8 0 0 1",
-	"2 3 fng 8 0 0 2",
-	"3 3 fng 8 0 0 3",
-	"4 3 fng 8 0 0 4",
-	"5 3 fng 8 0 0 5",
-	"plus 3 dplus 7 0 0",
-	"+ 3 dplus 7 0 0",
-	"accent 3 accent 8 0 0",
-	"> 3 accent 8 0 0",
-	"D.C. 3 dacs 16 0 0 D.C.",
-	"D.S. 3 dacs 16 0 0 D.S.",
-	"fine 3 dacs 16 0 0 FINE",
+	"0 3 fng 8 3 3 0",
+	"1 3 fng 8 3 3 1",
+	"2 3 fng 8 3 3 2",
+	"3 3 fng 8 3 3 3",
+	"4 3 fng 8 3 3 4",
+	"5 3 fng 8 3 3 5",
+	"plus 3 dplus 7 3 3",
+	"+ 3 dplus 7 3 3",
+	"accent 3 accent 8 4 4",
+	"> 3 accent 8 4 4",
+	"D.C. 3 dacs 16 10 10 D.C.",
+	"D.S. 3 dacs 16 10 10 D.S.",
+	"fine 3 dacs 16 10 10 FINE",
 	"f 6 pf 18 4 4",
 	"ff 6 pf 18 5 11",
 	"fff 6 pf 18 7 17",
 	"ffff 6 pf 18 8 24",
 	"mf 6 pf 18 5 11",
 	"mp 6 pf 18 5 11",
-	"mordent 3 lmrd 10 0 0",
-	"open 3 opend 10 0 0",
+	"mordent 3 lmrd 10 2 2",
+	"open 3 opend 10 2 2",
 	"p 6 pf 18 4 4",
 	"pp 6 pf 18 5 11",
 	"ppp 6 pf 18 7 17",
 	"pppp 6 pf 18 8 24",
-	"pralltriller 3 umrd 10 0 0",
+	"pralltriller 3 umrd 10 2 2",
 	"sfz 6 sfz 18 7 17",
-	"turn 3 turn 10 0 0",
-	"wedge 3 wedge 8 0 0",
-	"turnx 3 turnx 10 0 0",
+	"turn 3 turn 10 0 5",
+	"wedge 3 wedge 8 1 1",
+	"turnx 3 turnx 10 0 5",
 	"trill( 5 - 8 0 0",
 	"trill) 5 ltr 8 0 0",
-	"snap 3 snap 14 0 0",
-	"thumb 3 thumb 14 0 0",
+	"snap 3 snap 14 3 3",
+	"thumb 3 thumb 14 2 2",
 	"arpeggio 2 arp 12 10 0",
 	"crescendo( 7 - 18 0 0",
 	"crescendo) 7 cresc 18 0 0",
@@ -197,7 +197,6 @@ float y_get(struct SYMBOL *s,
 	    float w,
 	    float h)
 {
-#ifdef YSTEP
 	struct STAFF_S *p_staff;
 	int i, j;
 	float y;
@@ -231,71 +230,6 @@ i = 0;
 		}
 		y -= h;
 	}
-#else
-	struct SYMBOL *s2, *s_start;
-	int staff, start_seen;
-	float y;
-
-	s_start = s;
-	staff = s->staff;
-	if (s->x < x || s->ts_prev == 0)
-		start_seen = 1;
-	else {
-		while (s->ts_prev != 0 && s->ts_prev->x >= x)
-			s = s->ts_prev;
-		while (s->staff != staff) {
-			if (s->ts_prev == 0) {
-				while (s->staff != staff)
-					s = s->ts_next;
-				break;
-			}
-			s = s->ts_prev;
-		}
-		start_seen = s == s_start;
-	}
-	y = up ? s->dc_top : s->dc_bot;
-
-	if (x > s->x + 5) {
-/*fixme:KO if no decoration*/
-		if (up) {
-			y -= h;
-			if (y < s->ymx && x < s->x + 10)
-				y = s->ymx;
-		} else {
-			y += h;
-			if (y > s->ymn && x < s->x + 10)
-				y = s->ymn;
-		}
-		s = s->ts_next;
-	}
-	x += w;		/* right offset */
-	if (up) {
-		for (s2 = s; s2 != 0; s2 = s2->ts_next) {
-			if (start_seen) {
-				if (s2->x > x)
-					break;
-			} else if (s2 == s_start)
-				start_seen = 1;
-			if (s2->staff == staff) {
-				if (y < s2->dc_top)
-					y = s2->dc_top;
-			}
-		}
-	} else {
-		for (s2 = s; s2 != 0; s2 = s2->ts_next) {
-			if (start_seen) {
-				if (s2->x > x)
-					break;
-			} else if (s2 == s_start)
-				start_seen = 1;
-			if (s2->staff == staff) {
-				if (y > s2->dc_bot)
-					y = s2->dc_bot;
-			}
-		}
-		y -= h;
-	}
-#endif
 	return y;
 }
 
@@ -306,7 +240,6 @@ void y_set(struct SYMBOL *s,
 	   float w,
 	   float y)
 {
-#ifdef YSTEP
 	struct STAFF_S *p_staff;
 	int i, j;
 
@@ -336,45 +269,6 @@ i = 0;
 			i++;
 		}
 	}
-#else
-	struct SYMBOL *s_start;
-	int staff, start_seen;
-
-	s_start = s;
-	staff = s->staff;
-	if (x > s->x + 5) {
-		s = s->ts_next;
-		start_seen = 1;
-	} else {
-		while (s->ts_prev != 0 && s->ts_prev->x >= x)
-			s = s->ts_prev;
-		start_seen = 0;
-	}
-	x += w;		/* right offset */
-	if (up) {
-		for (; s != 0; s = s->ts_next) {
-			if (start_seen) {
-				if (s->x > x)
-					break;
-			} else if (s == s_start)
-				start_seen = 1;
-			if (s->staff == staff
-			    && s->dc_top < y)
-				s->dc_top = y;
-		}
-	} else {
-		for (; s != 0; s = s->ts_next) {
-			if (start_seen) {
-				if (s->x > x)
-					break;
-			} else if (s == s_start)
-				start_seen = 1;
-			if (s->staff == staff
-			    && s->dc_bot > y)
-				s->dc_bot = y;
-		}
-	}
-#endif
 }
 
 /* -- get the staff position of the dynamic marks -- */
@@ -405,12 +299,12 @@ static void d_arp(struct deco_elt *de)
 
 	s = de->s;
 	dd = &deco_def_tb[de->t];
-	xc = 10;
+	xc = 0;
 	for (m = 0; m <= s->nhd; m++) {
 		if (s->as.u.note.accs[m])
-			dx = 8 + s->shac[m];
+			dx = 5 + s->shac[m];
 		else {
-			dx = 10 - s->shhd[m];
+			dx = 6 - s->shhd[m];
 			switch (s->head) {
 			case H_SQUARE:
 			case H_OVAL:
@@ -513,18 +407,16 @@ static void d_near(struct deco_elt *de)
 	if (s->multi)
 		up = !up;
 	if (up)
-		y = s->ymx + 3;
-	else	y = s->ymn - 3;
-	if (y > -3 && y < 24 + 3) {
+		y = s->ymx;
+	else	y = s->ymn - dd->h;
+	if (y > -6 && y < 24) {
 		if (up)
-			y++;
-		y = (y + 5) / 6 * 6 - 3;	/* between lines */
+			y += 3;
+		y = (y + 6) / 6 * 6 - 6;	/* between lines */
 	}
-	if (s->ymx < y + dd->h)
+	if (up)
 		s->ymx = y + dd->h;
-	if (s->ymn > y - dd->h)
-		s->ymn = y - dd->h;
-
+	else	s->ymn = y;
 	de->x = s->x + s->shhd[s->stem >= 0 ? 0 : s->nhd];
 	de->y = (float) y;
 }
@@ -604,7 +496,7 @@ static void d_trill(struct deco_elt *de)
 	struct SYMBOL *s;
 	struct deco_def_s *dd;
 	int staff, up;
-	float x, y, dx;
+	float x, y, w;
 	struct SYMBOL *s2;
 
 	if (de->flags & DE_LDST)
@@ -626,32 +518,45 @@ static void d_trill(struct deco_elt *de)
 
 	up = s2->multi >= 0;
 	if (de->defl & DEF_NOEN) {	/* if no decoration end */
-		dx = de->x - x;
-		if (dx < 20) {
+		w = de->x - x;
+		if (w < 20) {
 			x = de->x - 20 - 3;
-			dx = 20;
+			w = 20;
 		}
 	} else {
-		dx = s2->x - x - 6;
+		w = s2->x - x - 6;
 		if (s2->type == NOTE)
-			dx -= 6;
-		if (dx < 20) {
-			x -= (20 - dx) * 0.5;
+			w -= 6;
+		if (w < 20) {
+			x -= (20 - w) * 0.5;
 			if (de->start == 0)
-				x -= (20 - dx) * 0.5;
-			dx = 20;
+				x -= (20 - w) * 0.5;
+			w = 20;
 		}
 	}
 
-	y = y_get(s2, up, x, dx, dd->h);
+	y = y_get(s2, up, x, w, dd->h);
+	if (up) {
+		float stafft;
+
+		stafft = staff_tb[s->staff].topbar + 2;
+		if (y < stafft)
+			y = stafft;
+	} else {
+		float staffb;
+
+		staffb = staff_tb[s->staff].botbar - 2;
+		if (y > staffb)
+			y = staffb;
+	}
 	de->flags &= ~DE_LDEN;
 	de->flags |= DE_VAL;
-	de->v = dx;
+	de->v = w;
 	de->x = x;
 	de->y = y;
 	if (up)
 		y += dd->h;
-	y_set(s2, up, x, dx, y);
+	y_set(s2, up, x, w, y);
 }
 
 /* above (or below) the staff */
@@ -659,66 +564,49 @@ static void d_upstaff(struct deco_elt *de)
 {
 	struct SYMBOL *s;
 	struct deco_def_s *dd;
-	float x, yc;
+	float x, yc, stafft, staffb, w;
 	int inv;
 
 	s = de->s;
 	dd = &deco_def_tb[de->t];
 	inv = 0;
 	x = s->x + s->shhd[s->stem >= 0 ? 0 : s->nhd];
+	w = dd->wl + dd->wr;
+	stafft = staff_tb[s->staff].topbar + 2;
+	staffb = staff_tb[s->staff].botbar - 2;
 	if (dd->str != 0)
 		de->str = dd->str == 255 ? dd->name : str_tb[dd->str];
 	switch (de->t) {
 	case 2:		/* roll */
 		if (s->multi < 0
 		    || (s->multi == 0 && s->stem > 0)) {
-#ifdef YSTEP
-			yc = y_get(s, 0, s->x, 0, 0);
-#else
-			yc = s->dc_bot;
-#endif
-			if (yc > -2)
-				yc = -2;
+			yc = y_get(s, 0, s->x - dd->wl, w, 0);
+			if (yc > staffb)
+				yc = staffb;
 			yc -= dd->h;
-#ifdef YSTEP
 			y_set(s, 0, s->x, 0, yc);
-#else
-			s->dc_bot = yc;
-#endif
 			inv = 1;
 		} else {
-#ifdef YSTEP
 			yc = y_get(s, 1, s->x, 0, 0) + 3;
-#else
-			yc = s->dc_top + 3;
-#endif
-			if (yc < 24 + 2)
-				yc = 24 + 2;
+			if (yc < stafft)
+				yc = stafft;
 			if (s->stem <= 0
 			    && (s->dots == 0 || ((int) s->y % 6)))
 				yc -= 2;
-#ifdef YSTEP
-			y_set(s, 1, s->x, 0, yc + dd->h);
-#else
-			s->dc_top = yc + dd->h;
-#endif
+			y_set(s, 1, s->x - dd->wl, w, yc + dd->h);
 		}
 		break;
 	case 15:	/* breath */
 	case 16:	/* longphrase */
 	case 17:	/* mediumphrase */
 	case 18: {	/* shortphrase */
-		int stime, seq;
-
-		yc = 24 + 3;
-		stime = s->time;
-		seq = s->seq;
+		yc = stafft + 1;
 		for (s = s->ts_next; s != 0; s = s->ts_next)
-			if (s->time != stime || s->seq != seq)
+			if (s->shrink != 0)
 				break;
 		if (s != 0)
 			x += (s->x - x) * 0.4;
-		else	x += 10;
+		else	x += (realwidth - x) * 0.4;
 		break;
 	    }
 	case 20:	/* invertedturn */
@@ -729,32 +617,16 @@ static void d_upstaff(struct deco_elt *de)
 		if (s->multi >= 0
 		    && de->t != 19	/* invertedfermata */
 		    && !(de->flags & DE_BELOW)) {
-#ifdef YSTEP
-			yc = y_get(s, 1, s->x, 0, 0);
-#else
-			yc = s->dc_top;
-#endif
-			if (yc < 24 + 2)
-				yc = 24 + 2;
-#ifdef YSTEP
-			y_set(s, 1, s->x, 0, yc + dd->h);
-#else
-			s->dc_top = yc + dd->h;
-#endif
+			yc = y_get(s, 1, s->x - dd->wl, w, 0);
+			if (yc < stafft)
+				yc = stafft;
+			y_set(s, 1, s->x - dd->wl, w, yc + dd->h);
 		} else {
-#ifdef YSTEP
-			yc = y_get(s, 0, s->x, 0, 0);
-#else
-			yc = s->dc_bot;
-#endif
-			if (yc > -2)
-				yc = -2;
+			yc = y_get(s, 0, s->x - dd->wl, w, 0);
+			if (yc > staffb)
+				yc = staffb;
 			yc -= dd->h;
-#ifdef YSTEP
-			y_set(s, 0, s->x, 0, yc);
-#else
-			s->dc_bot = yc;
-#endif
+			y_set(s, 0, s->x - dd->wl, w, yc);
 			switch (de->t) {
 			case 3:		/* fermata */
 			case 19:	/* invertedfermata */
@@ -894,11 +766,7 @@ void deco_cnv(struct deco *dc,
 		default:
 			continue;
 		case 32:		/* 32 = invisible */
-			if (s->type != NOTE)
-				error(1, s,
-				      "Cannot have +%s+ on a rest or a bar",
-				       dd->name);
-			else	s->as.u.note.invis = 1;
+			s->as.flags |= ABC_F_INVIS;
 			break;
 		case 33:		/* 33 = beamon */
 			s->sflags |= S_BEAM_ON;
@@ -912,9 +780,9 @@ void deco_cnv(struct deco *dc,
 			}
 			s->sflags |= S_TREM;
 			s->sflags &= ~S_WORD_ST;
-			s->as.u.note.word_end = 1;
+			s->as.flags |= ABC_F_WORD_END;
 			s->prev->sflags |= (S_TREM | S_WORD_ST);
-			s->prev->as.u.note.word_end = 0;
+			s->prev->as.flags &= ~ABC_F_WORD_END;
 			s->nflags = s->prev->nflags = dd->name[4] - '0';
 			for (j = 0; j <= s->nhd; j++)
 				s->as.u.note.lens[j] *= 2;
@@ -1042,11 +910,13 @@ float deco_width(struct SYMBOL *s)
 				wl = 7;
 			break;
 		case 2:			/* arpeggio */
-			if (wl < 10)
-				wl = 10;
+			if (wl < 14)
+				wl = 14;
 			break;
 		}
 	}
+	if (wl != 0 && s->prev->type == BAR)
+		wl -= 3;
 	return wl;
 }
 
@@ -1277,7 +1147,7 @@ static void deco_create(struct SYMBOL *s,
 		de->s = s;
 		de->t = dd - deco_def_tb;
 		de->staff = s->staff;
-		if (s->type == NOTE && s->as.u.note.grace)
+		if (s->type == NOTE && (s->as.flags & ABC_F_GRACE))
 			de->flags = DE_GRACE;
 		l = strlen(dd->name) - 1;
 		if (l > 0) {
@@ -1358,17 +1228,23 @@ void draw_deco_note(void)
 {
 	struct deco_elt *de, *de2;
 	struct deco_def_s *dd;
-	int f, t, voice;
+	int f, t, staff, voice;
 
 	for (de = deco_head; de != 0; de = de->next) {
 		t = de->t;
 		dd = &deco_def_tb[t];
 		if (de->flags & DE_LDST) {	/* start of long decoration */
 			t++;
-			voice = de->s->voice;
+			voice = de->s->voice;	/* search in the voice */
 			for (de2 = de->next; de2 != 0; de2 = de2->next)
 				if (de2->t == t && de2->s->voice == voice)
 					break;
+			if (de2 == 0) {		/* search in the staff */
+				staff = de->s->staff;
+				for (de2 = de->next; de2 != 0; de2 = de2->next)
+					if (de2->t == t && de2->s->staff == staff)
+						break;
+			}
 			if (de2 == 0) {		/* no end, insert one */
 				de2 = (struct deco_elt *) getarena(sizeof *de2);
 				memset(de2, 0, sizeof *de2);
@@ -1446,7 +1322,6 @@ void draw_deco_staff(void)
 			if (minmax[i].ymax < 34)
 				minmax[i].ymax = 34;
 		}
-		strcf = -1;
 		for (s = first_voice->sym; s != 0; s = s->ts_next) {
 			if (s->as.text == 0)
 				continue;
@@ -1475,7 +1350,7 @@ void draw_deco_staff(void)
 			continue;
 
 		/* search the max y offset */
-		y = 24 + 6 + 20;
+		y = staff_tb[p_voice->staff].topbar + 6 + 20;
 		first_repeat = 0;
 		for (s = p_voice->sym->next; s != 0; s = s->next) {
 			if (s->type != BAR
@@ -1485,7 +1360,7 @@ void draw_deco_staff(void)
 			if (s->next == 0)
 				break;
 			if (first_repeat == 0) {
-				set_font(&cfmt.repeatfont);
+				set_font(REPEATFONT);
 				first_repeat = s;
 			}
 			s1 = s;
@@ -1525,38 +1400,12 @@ void draw_deco_staff(void)
 
 			/* have room for the repeat numbers */
 			if (s1->as.text != 0) {
-#ifdef YSTEP
-				w = cwid('W') * cfmt.repeatfont.swfac
+				w = cwid('W') * cfmt.font_tb[REPEATFONT].swfac
 					* strlen(s1->as.text);
 				y2 = y_get(s1, 1, s1->x + 4, w, 0);
-				y2 += cfmt.repeatfont.size + 2;
+				y2 += cfmt.font_tb[REPEATFONT].size + 2;
 				if (y < y2)
 					y = y2;
-#else
-			    for (s2 = s1->next; s2 != s; s2 = s2->next) {
-				switch (s2->type) {
-				case PART:
-				case TEMPO:
-				case FMTCHG:
-				case TUPLET:
-				case WHISTLE:
-					continue;
-				case GRACE:
-				case NOTE:
-				case REST:
-					y2 = s2->dc_top;
-					if (s2->as.text != 0)	/* if guitar chord */
-						y2 -= cfmt.font_tb[s->gcf].size; /* got already */
-					y2 += cfmt.repeatfont.size + 2;
-					if (y < y2)
-						y = y2;
-					break;
-				default:
-					break;
-				}
-				break;
-			    }
-#endif
 			}
 			if (s->as.u.bar.repeat_bar)
 				s = s->prev;
@@ -1590,28 +1439,28 @@ void draw_deco_staff(void)
 			if (s1 == s2)
 				break;
 			x = s1->x;
-			if ((s1->as.u.bar.type & 0x0f) == B_COL)
+			if ((s1->as.u.bar.type & 0x07) == B_COL)
 				x -= 4;
-			i = 0;			/* no bracket end */
-			w = s2->x - x - 8;
+			i = 0;				/* no bracket end */
 			if (s2->sflags & S_RBSTOP)
-				;
+				w = 8;			/* (w = left shift) */
 			else if (s2->type != BAR)
-				w = realwidth - x - 4;
+				w = s2->x - realwidth + 4;
 			else if (((s2->as.u.bar.type & 0xf0)	/* if complex bar */
 				   && s2->as.u.bar.type != (B_OBRA << 4) + B_CBRA)
 				 || s2->as.u.bar.type == B_CBRA) {
-				i =  2;		/* bracket start and stop */
-				if ((s2->as.u.bar.type & 0x0f) == B_COL)
-					w -= 4;
+				i =  2;			/* bracket start and stop */
+				if (s->staff > 0
+				    && !(staff_tb[s->staff - 1].flags[0] & STOP_BAR))
+					w = s2->wl;
+				else if ((s2->as.u.bar.type & 0x0f) == B_COL)
+					w = 12;
 				else if (!(s2->sflags & S_RRBAR)
-					 || s2->as.u.bar.type == B_CBRA
-					 || s2->as.u.bar.type == (B_CBRA << 4) + B_BAR)
-					w += 8;		/* explicit repeat end */
-				if (p_voice != first_voice
-				    && !staff_tb[s->staff - 1].stop_bar)
-					w -= 4;
-			}
+					 || s2->as.u.bar.type == B_CBRA)
+					w = 0;		/* explicit repeat end */
+				else	w = 8;
+			} else	w = 8;
+			w = s2->x - x - w;
 			p = s1->as.text;
 			if (p == 0) {
 				i--;		/* no bracket start (1) or not draw */
@@ -1625,7 +1474,7 @@ void draw_deco_staff(void)
 			}
 			if (i >= 0) {
 				PUT3("(%s)-%.1f %d ",
-				     p, cfmt.repeatfont.size * 0.8 + 1, i);
+				     p, cfmt.font_tb[REPEATFONT].size * 0.8 + 1, i);
 				putf(w);
 				putxy(x, y);
 				PUT1("y%d repbra\n", s1->staff);
@@ -1687,22 +1536,19 @@ void draw_deco_staff(void)
 static void draw_gchord(struct SYMBOL *s,
 			float gchy)
 {
-	float x, y, xspc, yspca, yspcc, gchyb, gchyl, gchyr;
+	float x, y, w, yspca, yspcc, gchyb, gchyl, gchyr;
 	float xmin, xmax, ymin, ymax;
 	int box;
 	char *p, *q, *psf, sep, antype;
+	char expf[16];
 
 	/* compute the y offset of all position types */
 	ymax = gchy;
 	yspcc = cfmt.font_tb[s->gcf].size;
 	yspca = cfmt.font_tb[s->anf].size;
-	gchy -= yspcc;
-#ifdef YSTEP
-	gchyb = y_get(s, 0, s->x, 3, 0) - yspca;
-#else
-	gchyb = s->dc_bot - yspca;
-#endif
-	gchyl = gchyr = s->yav - yspca * 0.75;
+	gchyb = y_get(s, 0, s->x, 3, 0);
+	gchyl = gchyr = s->yav + yspca * 0.25;
+	box = cfmt.gchordbox;
 	p = s->as.text;
 	antype = '\0';
 	sep = '\n';
@@ -1715,6 +1561,8 @@ static void draw_gchord(struct SYMBOL *s,
 		switch (antype) {
 		default:		/* guitar chord */
 			gchy += yspcc;
+			if (box)
+				gchy += 2;
 			break;
 		case '^':		/* above */
 			gchy += yspca;
@@ -1734,22 +1582,18 @@ static void draw_gchord(struct SYMBOL *s,
 			if (*p == '\0' || *p == ';' || *p == '\n')
 				break;
 			p++;
-			if (*p == ';' && p[-1] == '\\')
-				p++;
+			if (p[-1] == '\\') {
+				if (*p == '\\' || *p == ';')
+					p++;
+			}
 		}
 		sep = *p;
 		if (sep == '\0')
 			break;
 		p++;
 	}
-#ifdef YSTEP
-	ymin = gchy + yspcc;
-	y_set(s, 1, s->x, 3, ymin);
-#else
-	ymin = s->dc_top = gchy + yspcc;
-#endif
+	ymin = gchy;
 	xmin = xmax = s->x;
-	box = cfmt.gchordbox;
 	psf = 0;
 	x = y = 0;				/* (compiler warning) */
 
@@ -1781,82 +1625,103 @@ static void draw_gchord(struct SYMBOL *s,
 			else	antype = '\0';
 		}
 		for (q = p; ; q++) {
-			if (*q == ';' && q > p && q[-1] == '\\')
-				continue;
+			if (*q == '\\') {
+				q++;
+				if (*q == '\\' || *q == ';')
+					continue;
+			}
 			if (*q == '\0' || *q == ';' || *q == '\n') {
 				sep = *q;
 				*q = '\0';
 				break;
 			}
 		}
+		w = tex_str(p);
 		if (antype == '\0') {
-			if (strcf != s->gcf) {
-				strcf = s->gcf;
-				set_font(&cfmt.font_tb[s->gcf]);
+			set_font(s->gcf);
+			if (strchr(tex_buf, '\t') == 0)
+				psf = "gc";
+			else {
+				struct SYMBOL *next;
+
+				x = realwidth;
+				next = s->next;
+				while (next != 0) {
+					switch (next->type) {
+					default:
+						continue;
+					case NOTE:
+					case REST:
+					case BAR:
+						x = next->x;
+						break;
+					}
+					break;
+				}
+				sprintf(expf, "%.2f gx", x - s->x);
+				psf = expf;
+				strcat(tex_buf, "\t");
 			}
-			psf = "gc";
 		} else {
-			if (strcf != s->anf) {
-				strcf = s->anf;
-				set_font(&cfmt.font_tb[s->anf]);
-			}
+			set_font(s->anf);
 			psf = "an";
 		}
-		xspc = tex_str(p);
 		switch (antype) {
 		case '^':		/* above */
 		case '_':		/* below */
 		default: {		/* default = above */
-			float tmp;
+			float xspc;
 
-			tmp = xspc;
+			xspc = w;
 			xspc *= GCHPRE;
 			if (xspc > 8)
 				xspc = 8;
 			x = s->x - xspc;
 			if (antype == '_') {
-				y = gchyb;
-#ifdef YSTEP
-				y_set(s, 0, x, xspc, gchyb - 2);
-#else
-				s->dc_bot = gchyb - 2;
-#endif
 				gchyb -= yspca;
+				y = gchyb;
+				y_set(s, 0, x, w, gchyb - 2);
 			} else {
-				y = gchy;
+				if (!box)
+					y_set(s, 1, x, w, gchy);
 				if (antype == '\0') {
-					gchy -= yspcc;
+					y = gchy - yspcc;
 					if (box) {
 						if (x < xmin)
 							xmin = x;
-						tmp += x;
-						if (tmp > xmax)
-							xmax = tmp;
-						if (ymax < y + yspcc)
-							ymax = y + yspcc;
+						w += x;
+						if (w > xmax)
+							xmax = w;
+						gchy -= 1;
+						if (ymax < gchy)
+							ymax = gchy;
+						y -= 1;
 						if (ymin > y)
 							ymin = y;
+						gchy = y - 1;
 						box = 2;
-					}
-				} else	gchy -= yspca;
-
+					} else	gchy = y;
+				} else {
+					gchy -= yspca;
+					y = gchy;
+				}
 			}
 			break;
 		    }
 		case '<':		/* left */
 /*fixme: what symbol space?*/
-			x = s->x - xspc - 6;
+			x = s->x - w - 6;
 			if (s->as.u.note.accs[0])
 				x -= s->shac[0];
-			y = gchyl;
 			gchyl -= yspca;
+			y = gchyl;
 			break;
 		case '>':		/* right */
 			x = s->x + s->xmx + 6;
 			if (s->dots > 0)
 				x += 1.5 + 3.5 * s->dots;
-			y = gchyr;
 			gchyr -= yspca;
+			y = gchyr;
 			break;
 		case '@':		/* absolute */
 			antype = '\001';
@@ -1866,8 +1731,7 @@ static void draw_gchord(struct SYMBOL *s,
 			break;
 		}
 		putxy(x, y);
-		PUT3("y%d M(%s)%sshow ",
-		     s->staff, tex_buf, psf);
+		PUT3("y%d M(%s)%sshow ", s->staff, tex_buf, psf);
 		if (sep == '\0')
 			break;
 		*q = sep;
@@ -1876,9 +1740,12 @@ static void draw_gchord(struct SYMBOL *s,
 
 	/* draw the box */
 	if (box == 2) {		/* if any normal guitar chord */
+		xmin -= 2;
+		w = xmax - xmin + 2;
+		y_set(s, 1, xmin, w, ymax + 2);
 		PUT5("%.1f %.1f y%d %.1f %.1f box",
-		     xmin - 2, ymin - 5, s->staff,
-		     xmax - xmin + 4, ymax - ymin + 4);
+		     xmin, ymin - 5, s->staff,
+		     w, ymax - ymin + 4);
 	}
 	PUT0("\n");
 }
@@ -1929,22 +1796,22 @@ void draw_measnb(void)
 			s = s->prev;
 		if (cfmt.measurenb == 0) {
 			s = s->prev;		/* clef */
-			set_font(&cfmt.measurefont);
+			set_font(MEASUREFONT);
 			any_nb = 1;
 			x = 0;
 			w = 20;
 			y = y_get(s, 1, x, w, 0);
-			if (y < 24 + 14)
-				y = 24 + 14;
+			if (y < staff_tb[0].topbar + 14)
+				y = staff_tb[0].topbar + 14;
 			PUT0("0 ");
 			puty(y);
 			PUT2("y0 M(%d)%s",nbar, showm);
-			y_set(s, 1, x, w, y + cfmt.measurefont.size + 2);
+			y_set(s, 1, x, w, y + cfmt.font_tb[MEASUREFONT].size + 2);
 		} else if (nbar % cfmt.measurenb == 0) {
-			x = s->x - 8;
-			set_font(&cfmt.measurefont);
+			x = s->x - s->wl;
+			set_font(MEASUREFONT);
 			any_nb = 1;
-			w = cwid('0') * cfmt.measurefont.size;
+			w = cwid('0') * cfmt.font_tb[MEASUREFONT].size;
 			if (nbar >= 10) {
 				if (nbar >= 100)
 					w *= 3;
@@ -1953,11 +1820,14 @@ void draw_measnb(void)
 			if (cfmt.measurebox)
 				w += 4;
 			y = y_get(s, 1, x, w, 0);
-			if (y < 24 + 7)
-				y = 24 + 7;
+			if (y < staff_tb[0].topbar + 6)
+				y = staff_tb[0].topbar + 6;
+			y += 2;
 			putxy(x, y);
 			PUT2("y0 M(%d)%s", nbar, showm);
-			y_set(s, 1, x, w, y + cfmt.measurefont.size + 2);
+			y += cfmt.font_tb[MEASUREFONT].size;
+			y_set(s, 1, x, w, y);
+			s->ymx = y;
 		}
 	}
 
@@ -1973,10 +1843,6 @@ void draw_measnb(void)
 		case MREST:
 			nbar += s->as.u.bar.len - 1;
 			continue;
-		case MREP:
-			if (s->as.u.bar.len > 2)
-				nbar += s->as.u.bar.len - 2;
-			continue;
 		default:
 			continue;
 		case BAR:
@@ -2001,15 +1867,13 @@ void draw_measnb(void)
 		    || s->next == 0
 		    || cfmt.measurenb == 0
 		    || (nbar % cfmt.measurenb) != 0
-		    || nbar <= 1
-		    || (s->next->type == MREP
-			&& s->next->as.u.bar.len > 1))
+		    || nbar <= 1)
 			continue;
 		if (!any_nb) {
 			any_nb = 1;
-			set_font(&cfmt.measurefont);
+			set_font(MEASUREFONT);
 		}
-		w = cwid('0') * cfmt.measurefont.size;
+		w = cwid('0') * cfmt.font_tb[MEASUREFONT].size;
 		if (nbar >= 10) {
 			if (nbar >= 100)
 				w *= 3;
@@ -2019,21 +1883,24 @@ void draw_measnb(void)
 			w += 4;
 		x = s->x - w * 0.4;
 		y = y_get(s, 1, x, w, 0);
-		if (y < 24 + 7)
-			y = 24 + 7;
+		if (y < staff_tb[0].topbar + 6)
+			y = staff_tb[0].topbar + 6;
 		if (s->next->type == NOTE) {
 			if (s->next->stem > 0) {
-				if (y < s->next->ys - cfmt.measurefont.size)
-					y = s->next->ys - cfmt.measurefont.size;
+				if (y < s->next->ys - cfmt.font_tb[MEASUREFONT].size)
+					y = s->next->ys - cfmt.font_tb[MEASUREFONT].size;
 			} else {
 				if (y < s->next->y)
 					y = s->next->y;
 			}
 		}
+		y += 2;
 		PUT0(" ");
 		putxy(x, y);
 		PUT2("y0 M(%d)%s",nbar, showm);
-		y_set(s, 1, x, w, y + cfmt.measurefont.size + 2);
+		y += cfmt.font_tb[MEASUREFONT].size;
+		y_set(s, 1, x, w, y);
+		s->ymx = y;
 	}
 	if (any_nb)
 		PUT0("\n");
@@ -2064,8 +1931,8 @@ static void draw_notempo(struct SYMBOL *s, int len, float sc)
 	int head, dots, flags;
 	float dx;
 
-	identify_note(s, len, &head, &dots, &flags);
 	PUT1("gsave %.2f dup scale 8 3 RM currentpoint ", sc);
+	identify_note(s, len, &head, &dots, &flags);
 	switch (head) {
 	case H_OVAL:
 		PUT0("HD");
@@ -2111,6 +1978,64 @@ static void draw_notempo(struct SYMBOL *s, int len, float sc)
 	PUT1(" grestore %.1f 0 RM\n", (dx + 15) * sc);
 }
 
+/* -- return the tempo width -- */
+float tempo_width(struct SYMBOL *s)
+{
+	int i;
+	float w;
+
+	w = 0;
+	if (s->as.u.tempo.str1 != 0)
+		w += tex_str(s->as.u.tempo.str1);
+	if (s->as.u.tempo.value != 0) {
+		i = 1;
+		while (i < sizeof s->as.u.tempo.length
+				/ sizeof s->as.u.tempo.length[0]
+		       && s->as.u.tempo.length[i] > 0) {
+			w += 10;
+			i++;
+		}
+		w += 6 + cwid(' ') * cfmt.font_tb[TEMPOFONT].size * 6
+			+ 10 + 10;
+	}
+	if (s->as.u.tempo.str2 != 0)
+		w += tex_str(s->as.u.tempo.str2);
+	return w;
+}
+
+/* - output a tempo --*/
+void write_tempo(struct SYMBOL *s,
+		 int beat,
+		 float sc)
+{
+	if (s->as.u.tempo.str1 != 0)
+		put_str(s->as.u.tempo.str1, A_LEFT);
+	if (s->as.u.tempo.value != 0) {
+		int j, top, bot;
+
+		sc *= 0.7 * cfmt.font_tb[TEMPOFONT].size / 15.0;	/*fixme: 15.0 = initial tempofont*/
+		if (s->as.u.tempo.length[0] == 0) {
+			if (beat == 0)
+				beat = get_beat(&first_voice->meter);
+			s->as.u.tempo.length[0] = beat;
+		}
+		for (j = 0;
+		     j < sizeof s->as.u.tempo.length
+				/ sizeof s->as.u.tempo.length[0]
+			&& s->as.u.tempo.length[j] > 0;
+		     j++) {
+			draw_notempo(s, s->as.u.tempo.length[j], sc);
+		}
+		PUT0("(= )show ");
+		if (sscanf(s->as.u.tempo.value, "%d/%d", &top, &bot) == 2
+		    && bot > 0)
+			draw_notempo(s, top * BASE_LEN / bot, sc);
+		else	put_str(s->as.u.tempo.value, A_LEFT);
+	}
+	if (s->as.u.tempo.str2 != 0)
+		put_str(s->as.u.tempo.str2, A_LEFT);
+}
+
 /* -- draw the parts and the tempo information -- */
 /* (the staves are being defined) */
 float draw_partempo(float top,
@@ -2118,53 +2043,45 @@ float draw_partempo(float top,
 		    int any_tempo)
 {
 	struct SYMBOL *s;
-	float h, ht, w, y, ymin, nw, dy;
-	int i;
+	float h, ht, w, y, ymin, dy;
 
 	/* put the tempo indication at top */
 	dy = 0;
 	if (any_tempo) {
-		int beat;
-		float sc;
+		int beat, dosh, shift;
+		float x;
 
-		ht = cfmt.tempofont.size + 2 + 2;
-		str_font(&cfmt.tempofont);
-		nw = 6. + cwid(' ') * cfmt.tempofont.size * 6;
-/*fixme:have tempo on other voices but the 1st?*/
+		ht = cfmt.font_tb[TEMPOFONT].size + 2 + 2;
+		str_font(TEMPOFONT);
 
 		/* get the minimal y offset */
-		ymin = 24 + 12;
+		ymin = staff_tb[0].topbar + 12;
+		dosh = 0;
+		shift = 1;
+		x = 0;
+/*fixme:have tempo on other voices but the 1st?*/
 		for (s = first_voice->sym->next; s != 0; s = s->next) {
 			if (s->type != TEMPO)
 				continue;
-
-			if (s->as.u.tempo.str1 != 0)
-				nw += tex_str(s->as.u.tempo.str1);
-			if (s->as.u.tempo.value != 0) {
-				i = 1;
-				while (i < sizeof s->as.u.tempo.length
-						/ sizeof s->as.u.tempo.length[0]
-				       && s->as.u.tempo.length[i] > 0) {
-					nw += 10;
-					i++;
-				}
-				nw += 10 + 10;
-			}
-			if (s->as.u.tempo.str2 != 0)
-				nw += tex_str(s->as.u.tempo.str2);
-			y = y_get(s, 1, s->x - 5, nw + 30 + 10, 0) + 2;
+			w = tempo_width(s);
+			y = y_get(s, 1, s->x - 5, w, 0) + 2;
 			if (y > ymin)
 				ymin = y;
+			if (x >= s->x - 5 && !(dosh & (shift >> 1)))
+				dosh |= shift;
+			shift <<= 1;
+			x = s->x - 5 + w;
 		}
+		y = 2 - ht;
+		h = y - ht;
+		if (dosh != 0)
+			ht *= 2;
 		if (top < ymin + ht)
 			dy = ymin + ht - top;
 
 		/* draw the tempo indications */
-		str_font(&cfmt.tempofont);
-		set_font(&cfmt.tempofont);
-		strcf = 0;
-		sc = 0.7 * cfmt.tempofont.size / 15.0;	/*fixme: 15.0 = initial tempofont*/
-		beat = get_beat(&first_voice->meter);
+		set_font(TEMPOFONT);
+		beat = 0;
 		for (s = first_voice->sym; s != 0; s = s->next) {
 			if (s->type != TEMPO) {
 				if (s->type == TIMESIG)
@@ -2173,32 +2090,10 @@ float draw_partempo(float top,
 			}
 
 			/*fixme: cf left shift (-5)*/
-			PUT2("%.1f %.1f M ", s->x - 5, 2 - ht);
-			if (s->as.u.tempo.str1 != 0)
-				put_str(s->as.u.tempo.str1, A_LEFT);
-
-			/* draw the tempo indication, if specified */
-			if (s->as.u.tempo.value != 0) {
-				int j, top, bot;
-
-				if (s->as.u.tempo.length[0] == 0)
-					s->as.u.tempo.length[0] = beat;
-				for (j = 0;
-				     j < sizeof s->as.u.tempo.length
-						/ sizeof s->as.u.tempo.length[0]
-					&& s->as.u.tempo.length[j] > 0;
-				     j++) {
-					draw_notempo(s, s->as.u.tempo.length[j], sc);
-				}
-				PUT0("(= )show ");
-				if (sscanf(s->as.u.tempo.value, "%d/%d", &top, &bot) == 2
-				    && bot > 0)
-					draw_notempo(s, top * BASE_LEN / bot, sc);
-				else	put_str(s->as.u.tempo.value, A_LEFT);
-			}
-
-			if (s->as.u.tempo.str2 != 0)
-				put_str(s->as.u.tempo.str2, A_LEFT);
+			PUT2("%.1f %.1f M ", s->x - 5,
+				(dosh & 1) ? h : y);
+			dosh >>= 1;
+			write_tempo(s, beat, 1);
 		}
 	} else	ht = 0;
 
@@ -2207,10 +2102,9 @@ float draw_partempo(float top,
 		return dy;
 
 /*fixme: should reduce if parts don't overlap tempo...*/
-	h = cfmt.partsfont.size + 2 + 2;	/* + cfmt.partsspace; */
-	
-	str_font(&cfmt.partsfont);
-	ymin = 24 + 14;
+	h = cfmt.font_tb[PARTSFONT].size + 2 + 2;	/* + cfmt.partsspace; */
+	str_font(PARTSFONT);
+	ymin = staff_tb[0].topbar + 14;
 	for (s = first_voice->sym->next; s != 0; s = s->next) {
 		if (s->type != PART)
 			continue;
@@ -2222,20 +2116,20 @@ float draw_partempo(float top,
 	if (top < ymin + h + ht)
 		dy = ymin + h + ht - top;
 
-	set_font(&cfmt.partsfont);
+	set_font(PARTSFONT);
 	for (s = first_voice->sym->next; s != 0; s = s->next) {
 		if (s->type != PART)
 			continue;
 		tex_str(&s->as.text[2]);
 		PUT4("%.1f %.1f M(%s)show%s\n",
-		     s->x - 10., 2. - ht - h, tex_buf,
-		     cfmt.partsbox ? "b" : "");
+		     s->x - 10, 2 - ht - h,
+		     tex_buf, cfmt.partsbox ? "b" : "");
 	}
 	return dy;
 }
 
 /* -- initialize the default decorations -- */
-void reset_deco(int deco_old)
+void reset_deco(void)
 {
 	memset(&deco_glob, 0, sizeof deco_glob);
 
@@ -2262,7 +2156,7 @@ void reset_deco(int deco_old)
 	deco_glob['R'] = 2;
 
 	/* abc2ps */
-	if (deco_old) {
+	if (cfmt.abc2pscompat) {
 		deco_glob['M'] = 14;
 	}
 }
