@@ -3,7 +3,7 @@
  *
  * This file is part of abcm2ps.
  *
- * Copyright (C) 1998-2006 Jean-Franois Moine
+ * Copyright (C) 1998-2007 Jean-Franois Moine
  * Adapted from abc2ps, Copyright (C) 1996,1997 Michael Methfessel
  *
  * This program is free software; you can redistribute it and/or modify
@@ -26,7 +26,6 @@
 #include <string.h>
 #include <time.h>
 #include <ctype.h>
-#include <math.h>
 
 #include "abcparse.h"
 #include "abc2ps.h"
@@ -34,6 +33,7 @@
 struct FORMAT cfmt;		/* current format for output */
 
 char font_enc[MAXFONTS];		/* font encoding */
+static char def_font_enc[MAXFONTS];	/* default font encoding */
 static char *fontnames[MAXFONTS];	/* list of font names */
 static char used_font[MAXFONTS];	/* used fonts */
 static int nfontnames;
@@ -53,17 +53,17 @@ static struct format {
 	char subtype;		/* special cases - see code */
 	short lock;
 } format_tb[] = {
+	{"abc2pscompat", &cfmt.abc2pscompat, FORMAT_B, 0},
 	{"alignbars", &cfmt.alignbars, FORMAT_I, 0},
 	{"aligncomposer", &cfmt.aligncomposer, FORMAT_I, 0},
 	{"autoclef", &cfmt.autoclef, FORMAT_B, 0},
-	{"annotationfont", &cfmt.annotationfont, FORMAT_F, 0},
-	{"barnumbers", &cfmt.measurenb, FORMAT_I, 0},
+	{"annotationfont", &cfmt.font_tb[ANNOTATIONFONT], FORMAT_F, 0},
 	{"barsperstaff", &cfmt.barsperstaff, FORMAT_I, 0},
 	{"botmargin", &cfmt.botmargin, FORMAT_U, 0},
 	{"bstemdown", &cfmt.bstemdown, FORMAT_B, 0},
 	{"combinevoices", &cfmt.combinevoices, FORMAT_B, 0},
 	{"comball", &cfmt.comball, FORMAT_B, 0},
-	{"composerfont", &cfmt.composerfont, FORMAT_F, 0},
+	{"composerfont", &cfmt.font_tb[COMPOSERFONT], FORMAT_F, 0},
 	{"composerspace", &cfmt.composerspace, FORMAT_U, 0},
 	{"contbarnb", &cfmt.contbarnb, FORMAT_B, 0},
 	{"continueall", &cfmt.continueall, FORMAT_B, 0},
@@ -73,18 +73,19 @@ static struct format {
 	{"exprabove", &cfmt.exprabove, FORMAT_B, 0},
 	{"exprbelow", &cfmt.exprbelow, FORMAT_B, 0},
 	{"footer", &cfmt.footer, FORMAT_S, 0},
-	{"footerfont", &cfmt.footerfont, FORMAT_F, 0},
+	{"footerfont", &cfmt.font_tb[FOOTERFONT], FORMAT_F, 0},
 	{"freegchord", &cfmt.freegchord, FORMAT_B, 0},
 	{"flatbeams", &cfmt.flatbeams, FORMAT_B, 0},
 	{"gchordbox", &cfmt.gchordbox, FORMAT_B, 0},
-	{"gchordfont", &cfmt.gchordfont, FORMAT_F, 3},
+	{"gchordfont", &cfmt.font_tb[GCHORDFONT], FORMAT_F, 3},
 	{"graceslurs", &cfmt.graceslurs, FORMAT_B, 0},
+	{"gracespace", &cfmt.gracespace, FORMAT_I, 5},
 	{"header", &cfmt.header, FORMAT_S, 0},
-	{"headerfont", &cfmt.headerfont, FORMAT_F, 0},
-	{"historyfont", &cfmt.historyfont, FORMAT_F, 0},
+	{"headerfont", &cfmt.font_tb[HEADERFONT], FORMAT_F, 0},
+	{"historyfont", &cfmt.font_tb[HISTORYFONT], FORMAT_F, 0},
 	{"hyphencont", &cfmt.hyphencont, FORMAT_B, 0},
 	{"indent", &cfmt.indent, FORMAT_U, 0},
-	{"infofont", &cfmt.infofont, FORMAT_F, 0},
+	{"infofont", &cfmt.font_tb[INFOFONT], FORMAT_F, 0},
 	{"infoline", &cfmt.infoline, FORMAT_B, 0},
 	{"infospace", &cfmt.infospace, FORMAT_U, 0},
 	{"landscape", &cfmt.landscape, FORMAT_B, 0},
@@ -95,7 +96,7 @@ static struct format {
 	{"maxsysstaffsep", &cfmt.maxsysstaffsep, FORMAT_U, 0},
 	{"measurebox", &cfmt.measurebox, FORMAT_B, 0},
 	{"measurefirst", &cfmt.measurefirst, FORMAT_I, 2},
-	{"measurefont", &cfmt.measurefont, FORMAT_F, 2},
+	{"measurefont", &cfmt.font_tb[MEASUREFONT], FORMAT_F, 2},
 	{"measurenb", &cfmt.measurenb, FORMAT_I, 0},
 	{"musiconly", &cfmt.musiconly, FORMAT_B, 0},
 	{"musicspace", &cfmt.musicspace, FORMAT_U, 0},
@@ -105,12 +106,11 @@ static struct format {
 	{"pagewidth", &cfmt.pagewidth, FORMAT_U, 0},
 	{"parskipfac", &cfmt.parskipfac, FORMAT_R, 0},
 	{"partsbox", &cfmt.partsbox, FORMAT_B, 0},
-	{"partsfont", &cfmt.partsfont, FORMAT_F, 1},
+	{"partsfont", &cfmt.font_tb[PARTSFONT], FORMAT_F, 1},
 	{"partsspace", &cfmt.partsspace, FORMAT_U, 0},
 	{"printparts", &cfmt.printparts, FORMAT_B, 0},
 	{"printtempo", &cfmt.printtempo, FORMAT_B, 0},
-	{"pslevel", &cfmt.pslevel, FORMAT_I, 0},
-	{"repeatfont", &cfmt.repeatfont, FORMAT_F, 0},
+	{"repeatfont", &cfmt.font_tb[REPEATFONT], FORMAT_F, 0},
 	{"rightmargin", &cfmt.rightmargin, FORMAT_U, 0},
 	{"scale", &cfmt.scale, FORMAT_R, 0},
 	{"setdefl", &cfmt.setdefl, FORMAT_B, 0},
@@ -118,41 +118,44 @@ static struct format {
 	{"setfont-2", &cfmt.font_tb[2], FORMAT_F, 0},
 	{"setfont-3", &cfmt.font_tb[3], FORMAT_F, 0},
 	{"setfont-4", &cfmt.font_tb[4], FORMAT_F, 0},
-#if DFONT_MIN!= 5
-#	error Bad start of dynamic fonts
+#if FONT_UMAX!=5
+#	error Bad number of user fonts
 #endif
-	{"shifthnote", &cfmt.shifthnote, FORMAT_B, 0},
+	{"shifthnote", &cfmt.shiftunisson, FORMAT_B, 0},	/*to remove*/
+	{"shiftunisson", &cfmt.shiftunisson, FORMAT_B, 0},
 	{"slurheight", &cfmt.slurheight, FORMAT_R, 0},
 	{"splittune", &cfmt.splittune, FORMAT_B, 0},
 	{"squarebreve", &cfmt.squarebreve, FORMAT_B, 0},
+	{"staffnonote", &cfmt.staffnonote, FORMAT_B, 0},
 	{"staffsep", &cfmt.staffsep, FORMAT_U, 0},
 	{"staffwidth", &staffwidth, FORMAT_U, 1},
 	{"stemheight", &cfmt.stemheight, FORMAT_R, 0},
 	{"straightflags", &cfmt.straightflags, FORMAT_B, 0},
 	{"stretchlast", &cfmt.stretchlast, FORMAT_B, 0},
 	{"stretchstaff", &cfmt.stretchstaff, FORMAT_B, 0},
-	{"subtitlefont", &cfmt.subtitlefont, FORMAT_F, 0},
+	{"subtitlefont", &cfmt.font_tb[SUBTITLEFONT], FORMAT_F, 0},
 	{"subtitlespace", &cfmt.subtitlespace, FORMAT_U, 0},
 	{"sysstaffsep", &cfmt.sysstaffsep, FORMAT_U, 0},
-	{"tempofont", &cfmt.tempofont, FORMAT_F, 0},
-	{"textfont", &cfmt.textfont, FORMAT_F, 0},
+	{"tempofont", &cfmt.font_tb[TEMPOFONT], FORMAT_F, 0},
+	{"textfont", &cfmt.font_tb[TEXTFONT], FORMAT_F, 0},
 	{"textoption", &cfmt.textoption, FORMAT_I, 4},
 	{"textspace", &cfmt.textspace, FORMAT_U, 0},
 	{"titlecaps", &cfmt.titlecaps, FORMAT_B, 0},
-	{"titlefont", &cfmt.titlefont, FORMAT_F, 0},
+	{"titlefont", &cfmt.font_tb[TITLEFONT], FORMAT_F, 0},
 	{"titleformat", &cfmt.titleformat, FORMAT_S, 0},
 	{"titleleft", &cfmt.titleleft, FORMAT_B, 0},
 	{"titlespace", &cfmt.titlespace, FORMAT_U, 0},
+	{"titletrim", &cfmt.titletrim, FORMAT_B, 0},
 	{"timewarn", &cfmt.timewarn, FORMAT_B, 0},
 	{"topmargin", &cfmt.topmargin, FORMAT_U, 0},
 	{"topspace", &cfmt.topspace, FORMAT_U, 0},
 	{"tuplets", &cfmt.tuplets, FORMAT_I, 3},
 	{"vocalabove", &cfmt.vocalabove, FORMAT_B, 0},
-	{"vocalfont", &cfmt.vocalfont, FORMAT_F, 0},
+	{"vocalfont", &cfmt.font_tb[VOCALFONT], FORMAT_F, 0},
 	{"vocalspace", &cfmt.vocalspace, FORMAT_U, 0},
-	{"voicefont", &cfmt.voicefont, FORMAT_F, 0},
+	{"voicefont", &cfmt.font_tb[VOICEFONT], FORMAT_F, 0},
 	{"withxrefs", &cfmt.withxrefs, FORMAT_B, 0},
-	{"wordsfont", &cfmt.wordsfont, FORMAT_F, 0},
+	{"wordsfont", &cfmt.font_tb[WORDSFONT], FORMAT_F, 0},
 	{"wordsspace", &cfmt.wordsspace, FORMAT_U, 0},
 	{"writehistory", &cfmt.writehistory, FORMAT_B, 0},
 	{0, 0, 0, 0}		/* end of table */
@@ -172,16 +175,25 @@ static char *enc_name[20] = {
 #endif
 };
 
-/* -- add a font -- */
-static int add_font(char *fname, int encoding)
+/* -- search a font and add it if not yet defined -- */
+static int get_font(char *fname, int encoding)
 {
 	int fnum;
 
+	/* get or set the default encoding */
 	for (fnum = nfontnames; --fnum >= 0; )
+		if (strcmp(fname, fontnames[fnum]) == 0) {
+			if (encoding < 0)
+				encoding = def_font_enc[fnum];
+			if (encoding == font_enc[fnum])
+				return fnum;		/* font found */
+		}
+	for ( ; --fnum >= 0; )
 		if (strcmp(fname, fontnames[fnum]) == 0
 		    && encoding == font_enc[fnum])
-			return fnum;		/* already here */
+			return fnum;
 
+	/* add the font */
 	if (nfontnames >= MAXFONTS) {
 		error(1, 0, "Too many fonts");
 		return 0;
@@ -191,6 +203,8 @@ static int add_font(char *fname, int encoding)
 		      "Cannot have a new font when the output file is opened");
 	fnum = nfontnames++;
 	fontnames[fnum] = strdup(fname);
+	if (encoding < 0)
+		encoding = cfmt.encoding;
 	font_enc[fnum] = encoding;
 	return fnum;
 }
@@ -200,14 +214,14 @@ static int dfont_set(struct FONTSPEC *f)
 {
 	int i;
 
-	for (i = DFONT_MIN; i < cfmt.ndfont; i++) {
+	for (i = FONT_DYN; i < cfmt.ndfont; i++) {
 		if (cfmt.font_tb[i].fnum == f->fnum
 		    && cfmt.font_tb[i].size == f->size)
 			return i;
 	}
-	if (i >= DFONT_MAX - 1) {
+	if (i >= FONT_MAX - 1) {
 		error(1, 0, "Too many dynamic fonts");
-		return DFONT_MAX - 1;
+		return FONT_MAX - 1;
 	}
 	memcpy(&cfmt.font_tb[i], f, sizeof cfmt.font_tb[0]);
 	cfmt.ndfont = i + 1;
@@ -221,7 +235,7 @@ static void fontspec(struct FONTSPEC *f,
 		     float size)
 {
 	if (name != 0)
-		f->fnum = add_font(name, encoding);
+		f->fnum = get_font(name, encoding);
 	else	name = fontnames[f->fnum];
 	f->size = size;
 	f->swfac = size;
@@ -236,16 +250,16 @@ static void fontspec(struct FONTSPEC *f,
 	else if (strncmp(name, "Courier", 7) == 0)
 		f->swfac *= 1.35;
 	else	f->swfac *= 1.2;		/* unknown font */
-	if (f == &cfmt.gchordfont)
+	if (f == &cfmt.font_tb[GCHORDFONT])
 		cfmt.gcf = dfont_set(f);
-	else if (f == &cfmt.annotationfont)
+	else if (f == &cfmt.font_tb[ANNOTATIONFONT])
 		cfmt.anf = dfont_set(f);
-	else if (f == &cfmt.vocalfont)
+	else if (f == &cfmt.font_tb[VOCALFONT])
 		cfmt.vof = dfont_set(f);
 }
 
-/* -- output the font definitions with their encodings -- */
-void define_fonts(void)
+/* -- output the encodings -- */
+void define_encodings(void)
 {
 	int i, enc;
 
@@ -261,6 +275,13 @@ void define_fonts(void)
 			define_encoding(i, enc_name[i]);
 		enc >>= 1;
 	}
+}
+
+/* -- output the font definitions with their encodings -- */
+void define_fonts(void)
+{
+	int i;
+
 	for (i = 0; i < nfontnames; i++) {
 		if (used_font[i])
 			define_font(fontnames[i], i, font_enc[i]);
@@ -273,23 +294,62 @@ void make_font_list(void)
 	struct FORMAT *f;
 
 	f = &cfmt;
-	used_font[f->annotationfont.fnum] = 1;
-	used_font[f->composerfont.fnum] = 1;
-	used_font[f->footerfont.fnum] = 1;
-	used_font[f->gchordfont.fnum] = 1;
-	used_font[f->headerfont.fnum] = 1;
-	used_font[f->historyfont.fnum] = 1;
-	used_font[f->infofont.fnum] = 1;
-	used_font[f->measurefont.fnum] = 1;
-	used_font[f->partsfont.fnum] = 1;
-	used_font[f->repeatfont.fnum] = 1;
-	used_font[f->subtitlefont.fnum] = 1;
-	used_font[f->tempofont.fnum] = 1;
-	used_font[f->textfont.fnum] = 1;
-	used_font[f->titlefont.fnum] = 1;
-	used_font[f->vocalfont.fnum] = 1;
-	used_font[f->voicefont.fnum] = 1;
-	used_font[f->wordsfont.fnum] = 1;
+	used_font[f->font_tb[ANNOTATIONFONT].fnum] = 1;
+	used_font[f->font_tb[COMPOSERFONT].fnum] = 1;
+	used_font[f->font_tb[FOOTERFONT].fnum] = 1;
+	used_font[f->font_tb[GCHORDFONT].fnum] = 1;
+	used_font[f->font_tb[HEADERFONT].fnum] = 1;
+	used_font[f->font_tb[HISTORYFONT].fnum] = 1;
+	used_font[f->font_tb[INFOFONT].fnum] = 1;
+	used_font[f->font_tb[MEASUREFONT].fnum] = 1;
+	used_font[f->font_tb[PARTSFONT].fnum] = 1;
+	used_font[f->font_tb[REPEATFONT].fnum] = 1;
+	used_font[f->font_tb[SUBTITLEFONT].fnum] = 1;
+	used_font[f->font_tb[TEMPOFONT].fnum] = 1;
+	used_font[f->font_tb[TEXTFONT].fnum] = 1;
+	used_font[f->font_tb[TITLEFONT].fnum] = 1;
+	used_font[f->font_tb[VOCALFONT].fnum] = 1;
+	used_font[f->font_tb[VOICEFONT].fnum] = 1;
+	used_font[f->font_tb[WORDSFONT].fnum] = 1;
+}
+
+/* -- set the name an information header type -- */
+/* the argument is
+ *	<letter> [ <possibly quoted string> ]
+ * this information is kept in the 'I' information */
+static void set_infoname(char *p)
+{
+	struct SYMBOL *s, *prev;
+
+	if (*p == 'I')
+		return;
+	s = info['I' - 'A'];
+	prev = 0;
+	while (s != 0) {
+		if (s->as.text[0] == *p)
+			break;
+		prev = s;
+		s = s->next;
+	}
+	if (p[1] == '\0') {		/* if delete */
+		if (s != 0) {
+			if ((prev->next = s->next) != 0)
+				prev->next->prev = prev;
+		}
+		return;
+	}
+	if (s == 0) {
+		s = (struct SYMBOL *) getarena(sizeof *s);
+		memset(s, 0, sizeof *s);
+		if (prev == 0)
+			info['I' - 'A'] = s;
+		else {
+			prev->next = s;
+			s->prev = prev;
+		}
+	}
+	s->as.text = (char *) getarena(strlen(p) + 1);
+	strcpy(s->as.text, p);
 }
 
 /* -- set the default format -- */
@@ -299,7 +359,6 @@ void set_format(void)
 
 	f = &cfmt;
 	memset(f, 0, sizeof *f);
-	f->ndfont = DFONT_MIN;
 	f->pageheight = PAGEHEIGHT;
 	f->pagewidth = PAGEWIDTH;
 	f->leftmargin = MARGIN;
@@ -327,33 +386,43 @@ void set_format(void)
 	f->parskipfac = 0.4;
 	f->measurenb = -1;
 	f->measurefirst = 1;
+	f->autoclef = 1;
+	f->dynalign = 1;
 	f->printparts = 1;
 	f->printtempo = 1;
-	f->autoclef = 1;
+	f->staffnonote = 1;
+	f->titletrim = 1;
 	f->aligncomposer = A_RIGHT;
-	f->dynalign = 1;
 	f->notespacingfactor = 1.414;
-	f->pslevel = 2;
 	f->stemheight = STEM;
 	f->dateformat = strdup("\\%b \\%e, \\%Y \\%H:\\%M");
-	f->textoption = OBEYLINES;
-	fontspec(&f->annotationfont, "Helvetica", 0, 12.0);
-	fontspec(&f->composerfont, "Times-Italic", 0, 14.0);
-	fontspec(&f->footerfont, "Times-Roman", 0, 12.0); /* not scaled */
-	fontspec(&f->gchordfont, "Helvetica", 0, 12.0);
-	fontspec(&f->headerfont, "Times-Roman", 0, 12.0); /* not scaled */
-	fontspec(&f->historyfont, "Times-Roman", 0, 16.0);
-	fontspec(&f->infofont,	"Times-Italic", 0, 14.0); /* same as composer by default */
-	fontspec(&f->measurefont, "Times-Italic", 0, 14.0);
-	fontspec(&f->partsfont, "Times-Roman", 0, 15.0);
-	fontspec(&f->repeatfont, "Times-Roman", 0, 13.0);
-	fontspec(&f->subtitlefont, "Times-Roman", 0, 16.0);
-	fontspec(&f->tempofont, "Times-Bold", 0, 15.0);
-	fontspec(&f->textfont,	"Times-Roman", 0, 16.0);
-	fontspec(&f->titlefont, "Times-Roman", 0, 20.0);
-	fontspec(&f->vocalfont, "Times-Bold", 0, 13.0);
-	fontspec(&f->voicefont, "Times-Bold", 0, 13.0);
-	fontspec(&f->wordsfont,	"Times-Roman", 0, 16.0);
+	f->gracespace = (65 << 16) | (80 << 8) | 120;	/* left-inside-right - unit 1/10 pt */
+	f->textoption = T_LEFT;
+	f->ndfont = FONT_DYN;
+	fontspec(&f->font_tb[ANNOTATIONFONT], "Helvetica", 0, 12.0);
+	fontspec(&f->font_tb[COMPOSERFONT], "Times-Italic", 0, 14.0);
+	fontspec(&f->font_tb[FOOTERFONT], "Times-Roman", 0, 12.0); /* not scaled */
+	fontspec(&f->font_tb[GCHORDFONT], "Helvetica", 0, 12.0);
+	fontspec(&f->font_tb[HEADERFONT], "Times-Roman", 0, 12.0); /* not scaled */
+	fontspec(&f->font_tb[HISTORYFONT], "Times-Roman", 0, 16.0);
+	fontspec(&f->font_tb[INFOFONT],	"Times-Italic", 0, 14.0); /* same as composer by default */
+	fontspec(&f->font_tb[MEASUREFONT], "Times-Italic", 0, 14.0);
+	fontspec(&f->font_tb[PARTSFONT], "Times-Roman", 0, 15.0);
+	fontspec(&f->font_tb[REPEATFONT], "Times-Roman", 0, 13.0);
+	fontspec(&f->font_tb[SUBTITLEFONT], "Times-Roman", 0, 16.0);
+	fontspec(&f->font_tb[TEMPOFONT], "Times-Bold", 0, 15.0);
+	fontspec(&f->font_tb[TEXTFONT],	"Times-Roman", 0, 16.0);
+	fontspec(&f->font_tb[TITLEFONT], "Times-Roman", 0, 20.0);
+	fontspec(&f->font_tb[VOCALFONT], "Times-Bold", 0, 13.0);
+	fontspec(&f->font_tb[VOICEFONT], "Times-Bold", 0, 13.0);
+	fontspec(&f->font_tb[WORDSFONT], "Times-Roman", 0, 16.0);
+	set_infoname("R \"Rhythm: \"");
+	set_infoname("B \"Book: \"");
+	set_infoname("S \"Source: \"");
+	set_infoname("D \"Discography: \"");
+	set_infoname("N \"Notes: \"");
+	set_infoname("Z \"Transcription: \"");
+	set_infoname("H \"History: \"");
 }
 
 /* -- print the current format -- */
@@ -379,6 +448,15 @@ static char yn[2][5]={"no","yes"};
 					(cfmt.tuplets >> 4) & 0x0f,
 					cfmt.tuplets & 0x0f);
 				break;
+			case 5:			/* gracespace */
+				printf("%d.%d %d.%d %d.%d\n",
+					(cfmt.gracespace >> 16) / 10,
+					(cfmt.gracespace >> 16) % 10,
+					((cfmt.gracespace >> 8) & 0xff) / 10,
+					((cfmt.gracespace >> 8) & 0xff) % 10,
+					(cfmt.gracespace & 0xff) / 10,
+					(cfmt.gracespace & 0xff) % 10);
+				break;
 			}
 			break;
 		case FORMAT_R:
@@ -454,7 +532,7 @@ int get_textopt(char *p)
 {
 	int option;
 
-	option = OBEYLINES;
+	option = T_LEFT;
 	if (*p == '\0'
 	    || strncmp(p, "obeylines", 9) == 0)
 		;
@@ -464,15 +542,17 @@ int get_textopt(char *p)
 	else if (strncmp(p, "ragged", 6) == 0
 		 || strncmp(p, "fill", 4) == 0)
 		option = T_FILL;
-	else if (strncmp(p, "center", 10) == 0)
-		option = OBEYCENTER;
+	else if (strncmp(p, "center", 6) == 0)
+		option = T_CENTER;
 	else if (strncmp(p, "skip", 4) == 0)
-		option = SKIP;
+		option = T_SKIP;
+	else if (strncmp(p, "right", 5) == 0)
+		option = T_RIGHT;
 	else	option = -1;
 	return option;
 }
 
-/* -- read a boolean value -- */
+/* -- get a boolean value -- */
 static int g_logv(char *p)
 {
 	switch (*p) {
@@ -497,15 +577,15 @@ static int g_logv(char *p)
 	return 0;
 }
 
-/* --  read a float variable, no units -- */
+/* --  get a float variable, no units -- */
 static float g_fltv(char *p)
 {
 	return atof(p);
 }
 
-/* -- read a font specifier -- */
+/* -- get a font specifier -- */
 static void g_fspc(char *p,
-		   struct FONTSPEC *fn)
+		   struct FONTSPEC *f)
 {
 	char fname[80];
 	int encoding;
@@ -514,20 +594,22 @@ static void g_fspc(char *p,
 	p = get_str(fname, p, sizeof fname);
 	if (!isdigit((unsigned char) *p)) {
 		if (*p == '*')
-			encoding = font_enc[fn->fnum];
+			encoding = font_enc[f->fnum];
 		else	encoding = get_encoding(p);
 		while (*p != '\0' && !isspace((unsigned char) *p))
 			p++;
 		while (isspace((unsigned char) *p))
 			p++;
-	} else	encoding = cfmt.encoding;
-	fsize = *p != '\0' ? g_fltv(p) : fn->size;
-	fontspec(fn,
+	} else	encoding = -1;
+	fsize = *p != '\0' && *p != '*' ? g_fltv(p) : f->size;
+	fontspec(f,
 		 strcmp(fname, "*") != 0 ? fname : 0,
 		 encoding,
 		 fsize);
 	if (!file_initialized)
-		used_font[fn->fnum] = 1;
+		used_font[f->fnum] = 1;
+	if (f - cfmt.font_tb == outft)
+		outft = -1;
 }
 
 /* -- parse a format line -- */
@@ -537,31 +619,54 @@ void interpret_fmt_line(char *w,		/* keyword */
 {
 	struct format *fd;
 
-	if (strcmp(w, "deco") == 0) {
-		deco_add(p);
-		return;
-	}
-	if (!strcmp(w, "font")) {
-		int fnum, encoding;
-		char fname[80];
+	switch (w[0]) {
+	case 'b':
+		if (strcmp(w, "barnumbers") == 0)
+			w = "measurenb";
+		break;
+	case 'd':
+		if (strcmp(w, "deco") == 0) {
+			deco_add(p);
+			return;
+		}
+		break;
+	case 'f':
+		if (strcmp(w, "font") == 0) {
+			int fnum, encoding;
+			char fname[80];
 
-		p = get_str(fname, p, sizeof fname);
-		if (*p == 0)
-			encoding = cfmt.encoding;
-		else	encoding = get_encoding(p);
-		fnum = add_font(fname, encoding);
-		used_font[fnum] = 1;
-		return;
-	}
-	if (strcmp(w, "format") == 0) {
-		if (read_fmt_file(p) < 0)
-			error(1, 0, "No such format file '%s'", p);
-		return;
-	}
-	if (strcmp(w, "postscript") == 0) {
-		if (!file_initialized)
-			user_ps_add(p);
-		return;
+			p = get_str(fname, p, sizeof fname);
+			if (*p == '\0')
+				encoding = cfmt.encoding;
+			else	encoding = get_encoding(p);
+			fnum = get_font(fname, encoding);
+			def_font_enc[fnum] = encoding;
+			used_font[fnum] = 1;
+			return;
+		}
+		if (strcmp(w, "format") == 0) {
+			if (read_fmt_file(p) < 0)
+				error(1, 0, "No such format file '%s'", p);
+			return;
+		}
+		break;
+	case 'i':
+		if (strcmp(w, "infoname") == 0) {
+			if (*p < 'A' || *p > 'Z') {
+				error(1, 0, "Bad info type '%c' in %%%%infoname",
+					*p);
+				return;
+			}
+			set_infoname(p);
+			return;
+		}
+		break;
+	case 'p':
+		if (strcmp(w, "postscript") == 0) {
+			if (!file_initialized)
+				user_ps_add(p);
+			return;
+		}
 	}
 	for (fd = format_tb; fd->name; fd++)
 		if (strcmp(w, fd->name) == 0)
@@ -573,19 +678,36 @@ void interpret_fmt_line(char *w,		/* keyword */
 	fd->lock |= lock;
 	switch (fd->type) {
 	case FORMAT_I:
-		if (fd->subtype == 3) {
+		if (fd->subtype == 3) {		/* tuplets */
 			unsigned i1, i2, i3;
 
 			if (sscanf(p, "%d %d %d", &i1, &i2, &i3) != 3
 			    || i1 > 2 || i2 > 2 || i3 > 2) {
 				error(1, 0,
-				      "Bad 'tuplets' value '%s' - ignored",
+				      "Bad 'tuplets' values '%s' - ignored",
 				      p);
 				return;
 			}
 			cfmt.tuplets = (i1 << 8) | (i2 << 4) | i3;
 			break;
 		}
+		if (fd->subtype == 5) {		/* gracespace */
+			unsigned i1, i2, i3;
+			float f1, f2, f3;
+
+			if (sscanf(p, "%f %f %f", &f1, &f2, &f3) != 3
+			    || f1 > 256 || f2 > 256 || f3 > 256) {
+				error(1, 0,
+				      "Bad 'gracespace' values '%s' - ignored",
+				      p);
+				return;
+			}
+			i1 = f2 * 10;
+			i2 = f2 * 10;
+			i3 = f3 * 10;
+			cfmt.gracespace = (i1 << 16) | (i2 << 8) | i3;
+			break;
+		}
 		if (fd->subtype == 1 && !isdigit(*p))	/* 'encoding' */
 			cfmt.encoding = get_encoding(p);
 		else if (fd->subtype == 4 && !isdigit(*p)) /* 'textoption' */
@@ -604,7 +726,8 @@ void interpret_fmt_line(char *w,		/* keyword */
 
 				for (i = 0; i < nfontnames; i++) {
 					if (font_enc[i] == 0)
-						font_enc[i] = cfmt.encoding;
+						def_font_enc[i] = font_enc[i]
+							= cfmt.encoding;
 				}
 			}
 			break;
@@ -616,7 +739,7 @@ void interpret_fmt_line(char *w,		/* keyword */
 				error(1, 0,
 				      "Bad 'textoption' value '%s'",
 				      p);
-				cfmt.textoption = OBEYLINES;
+				cfmt.textoption = T_LEFT;
 			}
 			break;
 		}
@@ -633,7 +756,6 @@ void interpret_fmt_line(char *w,		/* keyword */
 				cfmt.notespacingfactor = 1;
 				break;
 			}
-			dot_space = sqrt(cfmt.notespacingfactor);
 			i = C_XFLAGS;		/* crotchet index */
 			w = space_tb[i];
 			for ( ; --i >= 0; ) {
@@ -740,23 +862,58 @@ void lock_fmt(void *fmt)
 	fd->lock = 1;
 }
 
+/* -- open a file for reading -- */
+FILE *open_file(char *fn,	/* file name */
+		char *ext,	/* file type */
+		char *rfn)	/* real file name */
+{
+	FILE *fp;
+	char *p;
+	int l;
+
+	strcpy(rfn, fn);
+	if ((fp = fopen(rfn, "r")) != 0)
+		return fp;
+	strext(rfn, ext);
+	if ((fp = fopen(rfn, "r")) != 0)
+		return fp;
+	if (in_fname != 0
+	    && (p = strrchr(in_fname, DIRSEP)) != 0) {
+		l = p - in_fname + 1;
+		strncpy(rfn, in_fname, l);
+		strcpy(&rfn[l], fn);
+		if ((fp = fopen(rfn, "r")) != 0)
+			return fp;
+		strext(&rfn[l], ext);
+		if ((fp = fopen(rfn, "r")) != 0)
+			return fp;
+	}
+	if (*styd == '\0')
+		return 0;
+	sprintf(rfn, "%s%c%s", styd, DIRSEP, fn);
+	if ((fp = fopen(rfn, "r")) != 0)
+		return fp;
+	strext(rfn, ext);
+	if ((fp = fopen(rfn, "r")) != 0)
+		return fp;
+	return 0;
+}
+
 /* -- read a format file -- */
-int read_fmt_file(char *filename)
+int read_fmt_file(char *fn)
 {
 	FILE *fp;
 	char line[BSIZE], *p, *q;
 
-	strcpy(line, filename);
-	strext(line, "fmt");
-	if ((fp = fopen(line, "r")) == 0) {
-		if (*styd == '\0')
-			return -1;
-                sprintf(line, "%s%c%s", styd, DIRSEP, filename);
-		strext(line, "fmt");
-		if ((fp = fopen(line, "r")) == 0)
-			return -1;
+	line[0] = '\001';
+	if ((fp = open_file(fn, "fmt", &line[1])) == 0)
+		return -1;
+	if (strcmp(&line[strlen(line) - 3], ".ps") == 0) {
+		if (!file_initialized)
+			user_ps_add(line);
+		fclose(fp);
+		return 0;
 	}
-
 	for (;;) {
 		p = line;
 		if (!fgets(p, sizeof line, fp))
@@ -778,12 +935,8 @@ int read_fmt_file(char *filename)
 					continue;
 				if (strcmp(p, "endps") == 0)
 					break;
-#if 1
 				if (!file_initialized)
 					user_ps_add(p);
-#else
-				interpret_fmt_line("postscript", p, 0);
-#endif
 			}
 			continue;
 		}
@@ -804,19 +957,27 @@ int read_fmt_file(char *filename)
 }
 
 /* -- start a new font -- */
-void set_font(struct FONTSPEC *font)
+void set_font(int ft)
 {
 	int fnum;
+	struct FONTSPEC *f, *f2;
 
-	fnum = font->fnum;
+	if (ft == outft)
+		return;
+	f = &cfmt.font_tb[ft];
+	f2 = &cfmt.font_tb[outft];
+	outft = ft;
+	if (f->fnum == f2->fnum && f->size == f2->size)
+		return;
+	fnum = f->fnum;
 	if (!used_font[fnum]) {
 		error(1, 0,
 		      "Font \"%s\" not predefined; using first in list",
 		      fontnames[fnum]);
 		fnum = 0;
 	}
-	if (font->size == 0)
+	if (f->size == 0)
 		error(0, 0, "Font \"%s\" with a null size",
 		      fontnames[fnum]);
-	PUT2("%.1f F%d ", font->size, fnum);
+	PUT2("%.1f F%d ", f->size, fnum);
 }
